@@ -1,17 +1,13 @@
 import numpy as np
 from pandas import DataFrame
+
 try:
     from pandas.util import cache_readonly
 except ImportError:
     from pandas.util.decorators import cache_readonly
 
-from .pandas_vb_common import setup  # noqa
-
-
-class DataFrameAttributes(object):
-
-    goal_time = 0.2
 
+class DataFrameAttributes:
     def setup(self):
         self.df = DataFrame(np.random.randn(10, 6))
         self.cur_index = self.df.index
@@ -23,18 +19,17 @@ class DataFrameAttributes(object):
         self.df.index = self.cur_index
 
 
-class CacheReadonly(object):
-
-    goal_time = 0.2
-
+class CacheReadonly:
     def setup(self):
-
         class Foo:
-
             @cache_readonly
             def prop(self):
                 return 5
+
         self.obj = Foo()
 
     def time_cache_readonly(self):
         self.obj.prop
+
+
+from .pandas_vb_common import setup  # noqa: F401
