@@ -1,29 +1,23 @@
 import numpy as np
 import pandas.util.testing as tm
 from pandas import DataFrame, Series, MultiIndex, Timestamp, date_range
+
 try:
     from pandas.tseries.offsets import Nano, Hour
 except ImportError:
     # For compatibility with older versions
-    from pandas.core.datetools import * # noqa
-
-from .pandas_vb_common import setup # noqa
+    from pandas.core.datetools import *  # noqa
 
 
-class FromDicts(object):
-
-    goal_time = 0.2
-
+class FromDicts:
     def setup(self):
         N, K = 5000, 50
         self.index = tm.makeStringIndex(N)
         self.columns = tm.makeStringIndex(K)
-        frame = DataFrame(np.random.randn(N, K), index=self.index,
-                          columns=self.columns)
+        frame = DataFrame(np.random.randn(N, K), index=self.index, columns=self.columns)
         self.data = frame.to_dict()
-        self.dict_list = frame.to_dict(orient='records')
-        self.data2 = {i: {j: float(j) for j in range(100)}
-                      for i in range(2000)}
+        self.dict_list = frame.to_dict(orient="records")
+        self.data2 = {i: {j: float(j) for j in range(100)} for i in range(2000)}
 
     def time_list_of_dict(self):
         DataFrame(self.dict_list)
@@ -45,10 +39,7 @@ class FromDicts(object):
         DataFrame(self.data2)
 
 
-class FromSeries(object):
-
-    goal_time = 0.2
-
+class FromSeries:
     def setup(self):
         mi = MultiIndex.from_product([range(100), range(100)])
         self.s = Series(np.random.randn(10000), index=mi)
@@ -57,16 +48,15 @@ class FromSeries(object):
         DataFrame(self.s)
 
 
-class FromDictwithTimestamp(object):
+class FromDictwithTimestamp:
 
-    goal_time = 0.2
     params = [Nano(1), Hour(1)]
-    param_names = ['offset']
+    param_names = ["offset"]
 
     def setup(self, offset):
-        N = 10**3
+        N = 10 ** 3
         np.random.seed(1234)
-        idx = date_range(Timestamp('1/1/1900'), freq=offset, periods=N)
+        idx = date_range(Timestamp("1/1/1900"), freq=offset, periods=N)
         df = DataFrame(np.random.randn(N, 10), index=idx)
         self.d = df.to_dict()
 
@@ -74,11 +64,14 @@ class FromDictwithTimestamp(object):
         DataFrame(self.d)
 
 
-class FromRecords(object):
+class FromRecords:
 
-    goal_time = 0.2
     params = [None, 1000]
-    param_names = ['nrows']
+    param_names = ["nrows"]
+
+    # Generators get exhausted on use, so run setup before every call
+    number = 1
+    repeat = (3, 250, 10)
 
     def setup(self, nrows):
         N = 100000
@@ -89,13 +82,26 @@ class FromRecords(object):
         self.df = DataFrame.from_records(self.gen, nrows=nrows)
 
 
-class FromNDArray(object):
-
-    goal_time = 0.2
-
+class FromNDArray:
     def setup(self):
         N = 100000
         self.data = np.random.randn(N)
 
     def time_frame_from_ndarray(self):
         self.df = DataFrame(self.data)
+
+
+class FromLists:
+
+    goal_time = 0.2
+
+    def setup(self):
+        N = 1000
+        M = 100
+        self.data = [[j for j in range(M)] for i in range(N)]
+
+    def time_frame_from_lists(self):
+        self.df = DataFrame(self.data)
+
+
+from .pandas_vb_common import setup  # noqa: F401
