@@ -1,43 +1,35 @@
-import string
 import warnings
+import string
 
 import numpy as np
+
+from pandas import DataFrame, MultiIndex, NaT, Series, date_range, isnull, period_range
 import pandas.util.testing as tm
-from pandas import (DataFrame, Series, MultiIndex, date_range, period_range,
-                    isnull, NaT)
-
-from .pandas_vb_common import setup  # noqa
 
 
-class GetNumericData(object):
-
-    goal_time = 0.2
-
+class GetNumericData:
     def setup(self):
         self.df = DataFrame(np.random.randn(10000, 25))
-        self.df['foo'] = 'bar'
-        self.df['bar'] = 'baz'
-        with warnings.catch_warnings(record=True):
-            self.df = self.df.consolidate()
+        self.df["foo"] = "bar"
+        self.df["bar"] = "baz"
+        self.df = self.df._consolidate()
 
     def time_frame_get_numeric_data(self):
         self.df._get_numeric_data()
 
 
-class Lookup(object):
-
-    goal_time = 0.2
-
+class Lookup:
     def setup(self):
-        self.df = DataFrame(np.random.randn(10000, 8),
-                            columns=list('abcdefgh'))
-        self.df['foo'] = 'bar'
+        self.df = DataFrame(np.random.randn(10000, 8), columns=list("abcdefgh"))
+        self.df["foo"] = "bar"
         self.row_labels = list(self.df.index[::10])[:900]
         self.col_labels = list(self.df.columns) * 100
         self.row_labels_all = np.array(
-            list(self.df.index) * len(self.df.columns), dtype='object')
+            list(self.df.index) * len(self.df.columns), dtype="object"
+        )
         self.col_labels_all = np.array(
-            list(self.df.columns) * len(self.df.index), dtype='object')
+            list(self.df.columns) * len(self.df.index), dtype="object"
+        )
 
     def time_frame_fancy_lookup(self):
         self.df.lookup(self.row_labels, self.col_labels)
@@ -46,20 +38,22 @@ class Lookup(object):
         self.df.lookup(self.row_labels_all, self.col_labels_all)
 
 
-class Reindex(object):
-
-    goal_time = 0.2
-
+class Reindex:
     def setup(self):
-        N = 10**3
+        N = 10 ** 3
         self.df = DataFrame(np.random.randn(N * 10, N))
         self.idx = np.arange(4 * N, 7 * N)
         self.df2 = DataFrame(
-            {c: {0: np.random.randint(0, 2, N).astype(np.bool_),
-                 1: np.random.randint(0, N, N).astype(np.int16),
-                 2: np.random.randint(0, N, N).astype(np.int32),
-                 3: np.random.randint(0, N, N).astype(np.int64)}
-                [np.random.randint(0, 4)] for c in range(N)})
+            {
+                c: {
+                    0: np.random.randint(0, 2, N).astype(np.bool_),
+                    1: np.random.randint(0, N, N).astype(np.int16),
+                    2: np.random.randint(0, N, N).astype(np.int32),
+                    3: np.random.randint(0, N, N).astype(np.int64),
+                }[np.random.randint(0, 4)]
+                for c in range(N)
+            }
+        )
 
     def time_reindex_axis0(self):
         self.df.reindex(self.idx)
@@ -70,52 +64,142 @@ class Reindex(object):
     def time_reindex_both_axes(self):
         self.df.reindex(index=self.idx, columns=self.idx)
 
-    def time_reindex_both_axes_ix(self):
-        self.df.ix[self.idx, self.idx]
-
     def time_reindex_upcast(self):
         self.df2.reindex(np.random.permutation(range(1200)))
 
 
-class Iteration(object):
+class Rename:
+    def setup(self):
+        N = 10 ** 3
+        self.df = DataFrame(np.random.randn(N * 10, N))
+        self.idx = np.arange(4 * N, 7 * N)
+        self.dict_idx = {k: k for k in self.idx}
+        self.df2 = DataFrame(
+            {
+                c: {
+                    0: np.random.randint(0, 2, N).astype(np.bool_),
+                    1: np.random.randint(0, N, N).astype(np.int16),
+                    2: np.random.randint(0, N, N).astype(np.int32),
+                    3: np.random.randint(0, N, N).astype(np.int64),
+                }[np.random.randint(0, 4)]
+                for c in range(N)
+            }
+        )
 
-    goal_time = 0.2
+    def time_rename_single(self):
+        self.df.rename({0: 0})
+
+    def time_rename_axis0(self):
+        self.df.rename(self.dict_idx)
+
+    def time_rename_axis1(self):
+        self.df.rename(columns=self.dict_idx)
+
+    def time_rename_both_axes(self):
+        self.df.rename(index=self.dict_idx, columns=self.dict_idx)
+
+    def time_dict_rename_both_axes(self):
+        self.df.rename(index=self.dict_idx, columns=self.dict_idx)
+
+
+class Iteration:
+    # mem_itertuples_* benchmarks are slow
+    timeout = 120
 
     def setup(self):
         N = 1000
         self.df = DataFrame(np.random.randn(N * 10, N))
         self.df2 = DataFrame(np.random.randn(N * 50, 10))
-        self.df3 = DataFrame(np.random.randn(N, 5 * N),
-                             columns=['C' + str(c) for c in range(N * 5)])
+        self.df3 = DataFrame(
+            np.random.randn(N, 5 * N), columns=["C" + str(c) for c in range(N * 5)]
+        )
+        self.df4 = DataFrame(np.random.randn(N * 1000, 10))
 
-    def time_iteritems(self):
+    def time_items(self):
         # (monitor no-copying behaviour)
-        if hasattr(self.df, '_item_cache'):
+        if hasattr(self.df, "_item_cache"):
             self.df._item_cache.clear()
-        for name, col in self.df.iteritems():
+        for name, col in self.df.items():
             pass
 
-    def time_iteritems_cached(self):
-        for name, col in self.df.iteritems():
+    def time_items_cached(self):
+        for name, col in self.df.items():
             pass
 
     def time_iteritems_indexing(self):
         for col in self.df3:
             self.df3[col]
 
+    def time_itertuples_start(self):
+        self.df4.itertuples()
+
+    def time_itertuples_read_first(self):
+        next(self.df4.itertuples())
+
     def time_itertuples(self):
-        for row in self.df2.itertuples():
+        for row in self.df4.itertuples():
             pass
 
+    def time_itertuples_to_list(self):
+        list(self.df4.itertuples())
+
+    def mem_itertuples_start(self):
+        return self.df4.itertuples()
+
+    def peakmem_itertuples_start(self):
+        self.df4.itertuples()
+
+    def mem_itertuples_read_first(self):
+        return next(self.df4.itertuples())
+
+    def peakmem_itertuples(self):
+        for row in self.df4.itertuples():
+            pass
+
+    def mem_itertuples_to_list(self):
+        return list(self.df4.itertuples())
+
+    def peakmem_itertuples_to_list(self):
+        list(self.df4.itertuples())
+
+    def time_itertuples_raw_start(self):
+        self.df4.itertuples(index=False, name=None)
+
+    def time_itertuples_raw_read_first(self):
+        next(self.df4.itertuples(index=False, name=None))
+
+    def time_itertuples_raw_tuples(self):
+        for row in self.df4.itertuples(index=False, name=None):
+            pass
+
+    def time_itertuples_raw_tuples_to_list(self):
+        list(self.df4.itertuples(index=False, name=None))
+
+    def mem_itertuples_raw_start(self):
+        return self.df4.itertuples(index=False, name=None)
+
+    def peakmem_itertuples_raw_start(self):
+        self.df4.itertuples(index=False, name=None)
+
+    def peakmem_itertuples_raw_read_first(self):
+        next(self.df4.itertuples(index=False, name=None))
+
+    def peakmem_itertuples_raw(self):
+        for row in self.df4.itertuples(index=False, name=None):
+            pass
+
+    def mem_itertuples_raw_to_list(self):
+        return list(self.df4.itertuples(index=False, name=None))
+
+    def peakmem_itertuples_raw_to_list(self):
+        list(self.df4.itertuples(index=False, name=None))
+
     def time_iterrows(self):
         for row in self.df.iterrows():
             pass
 
 
-class ToString(object):
-
-    goal_time = 0.2
-
+class ToString:
     def setup(self):
         self.df = DataFrame(np.random.randn(100, 10))
 
@@ -123,24 +207,18 @@ class ToString(object):
         self.df.to_string()
 
 
-class ToHTML(object):
-
-    goal_time = 0.2
-
+class ToHTML:
     def setup(self):
         nrows = 500
         self.df2 = DataFrame(np.random.randn(nrows, 10))
-        self.df2[0] = period_range('2000', periods=nrows)
+        self.df2[0] = period_range("2000", periods=nrows)
         self.df2[1] = range(nrows)
 
     def time_to_html_mixed(self):
         self.df2.to_html()
 
 
-class Repr(object):
-
-    goal_time = 0.2
-
+class Repr:
     def setup(self):
         nrows = 10000
         data = np.random.randn(nrows, 10)
@@ -164,10 +242,7 @@ class Repr(object):
         repr(self.df_wide)
 
 
-class MaskBool(object):
-
-    goal_time = 0.2
-
+class MaskBool:
     def setup(self):
         data = np.random.randn(1000, 500)
         df = DataFrame(data)
@@ -182,12 +257,9 @@ class MaskBool(object):
         self.bools.astype(float).mask(self.mask)
 
 
-class Isnull(object):
-
-    goal_time = 0.2
-
+class Isnull:
     def setup(self):
-        N = 10**3
+        N = 10 ** 3
         self.df_no_null = DataFrame(np.random.randn(N, N))
 
         sample = np.array([np.nan, 1.0])
@@ -198,8 +270,20 @@ class Isnull(object):
         data = np.random.choice(sample, (N, N))
         self.df_strings = DataFrame(data)
 
-        sample = np.array([NaT, np.nan, None, np.datetime64('NaT'),
-                           np.timedelta64('NaT'), 0, 1, 2.0, '', 'abcd'])
+        sample = np.array(
+            [
+                NaT,
+                np.nan,
+                None,
+                np.datetime64("NaT"),
+                np.timedelta64("NaT"),
+                0,
+                1,
+                2.0,
+                "",
+                "abcd",
+            ]
+        )
         data = np.random.choice(sample, (N, N))
         self.df_obj = DataFrame(data)
 
@@ -216,11 +300,10 @@ class Isnull(object):
         isnull(self.df_obj)
 
 
-class Fillna(object):
+class Fillna:
 
-    goal_time = 0.2
-    params = ([True, False], ['pad', 'bfill'])
-    param_names = ['inplace', 'method']
+    params = ([True, False], ["pad", "bfill"])
+    param_names = ["inplace", "method"]
 
     def setup(self, inplace, method):
         values = np.random.randn(10000, 100)
@@ -231,19 +314,19 @@ class Fillna(object):
         self.df.fillna(inplace=inplace, method=method)
 
 
-class Dropna(object):
+class Dropna:
 
-    goal_time = 0.2
-    params = (['all', 'any'], [0, 1])
-    param_names = ['how', 'axis']
+    params = (["all", "any"], [0, 1])
+    param_names = ["how", "axis"]
 
     def setup(self, how, axis):
         self.df = DataFrame(np.random.randn(10000, 1000))
-        self.df.ix[50:1000, 20:50] = np.nan
-        self.df.ix[2000:3000] = np.nan
-        self.df.ix[:, 60:70] = np.nan
+        with warnings.catch_warnings(record=True):
+            self.df.ix[50:1000, 20:50] = np.nan
+            self.df.ix[2000:3000] = np.nan
+            self.df.ix[:, 60:70] = np.nan
         self.df_mixed = self.df.copy()
-        self.df_mixed['foo'] = 'bar'
+        self.df_mixed["foo"] = "bar"
 
     def time_dropna(self, how, axis):
         self.df.dropna(how=how, axis=axis)
@@ -252,28 +335,28 @@ class Dropna(object):
         self.df_mixed.dropna(how=how, axis=axis)
 
 
-class Count(object):
-
-    goal_time = 0.2
+class Count:
 
     params = [0, 1]
-    param_names = ['axis']
+    param_names = ["axis"]
 
     def setup(self, axis):
         self.df = DataFrame(np.random.randn(10000, 1000))
-        self.df.ix[50:1000, 20:50] = np.nan
-        self.df.ix[2000:3000] = np.nan
-        self.df.ix[:, 60:70] = np.nan
+        with warnings.catch_warnings(record=True):
+            self.df.ix[50:1000, 20:50] = np.nan
+            self.df.ix[2000:3000] = np.nan
+            self.df.ix[:, 60:70] = np.nan
         self.df_mixed = self.df.copy()
-        self.df_mixed['foo'] = 'bar'
+        self.df_mixed["foo"] = "bar"
 
         self.df.index = MultiIndex.from_arrays([self.df.index, self.df.index])
-        self.df.columns = MultiIndex.from_arrays([self.df.columns,
-                                                  self.df.columns])
-        self.df_mixed.index = MultiIndex.from_arrays([self.df_mixed.index,
-                                                      self.df_mixed.index])
-        self.df_mixed.columns = MultiIndex.from_arrays([self.df_mixed.columns,
-                                                        self.df_mixed.columns])
+        self.df.columns = MultiIndex.from_arrays([self.df.columns, self.df.columns])
+        self.df_mixed.index = MultiIndex.from_arrays(
+            [self.df_mixed.index, self.df_mixed.index]
+        )
+        self.df_mixed.columns = MultiIndex.from_arrays(
+            [self.df_mixed.columns, self.df_mixed.columns]
+        )
 
     def time_count_level_multi(self, axis):
         self.df.count(axis=axis, level=1)
@@ -282,16 +365,13 @@ class Count(object):
         self.df_mixed.count(axis=axis, level=1)
 
 
-class Apply(object):
-
-    goal_time = 0.2
-
+class Apply:
     def setup(self):
         self.df = DataFrame(np.random.randn(1000, 100))
 
         self.s = Series(np.arange(1028.0))
         self.df2 = DataFrame({i: self.s for i in range(1028)})
-        self.df3 = DataFrame(np.random.randn(1000, 3), columns=list('ABC'))
+        self.df3 = DataFrame(np.random.randn(1000, 3), columns=list("ABC"))
 
     def time_apply_user_func(self):
         self.df2.apply(lambda x: np.corrcoef(x, self.s)[(0, 1)])
@@ -309,13 +389,10 @@ class Apply(object):
         self.df.apply(lambda x: x)
 
     def time_apply_ref_by_name(self):
-        self.df3.apply(lambda x: x['A'] + x['B'], axis=1)
+        self.df3.apply(lambda x: x["A"] + x["B"], axis=1)
 
 
-class Dtypes(object):
-
-    goal_time = 0.2
-
+class Dtypes:
     def setup(self):
         self.df = DataFrame(np.random.randn(1000, 1000))
 
@@ -323,22 +400,19 @@ class Dtypes(object):
         self.df.dtypes
 
 
-class Equals(object):
-
-    goal_time = 0.2
-
+class Equals:
     def setup(self):
-        N = 10**3
+        N = 10 ** 3
         self.float_df = DataFrame(np.random.randn(N, N))
         self.float_df_nan = self.float_df.copy()
         self.float_df_nan.iloc[-1, -1] = np.nan
 
-        self.object_df = DataFrame('foo', index=range(N), columns=range(N))
+        self.object_df = DataFrame("foo", index=range(N), columns=range(N))
         self.object_df_nan = self.object_df.copy()
         self.object_df_nan.iloc[-1, -1] = np.nan
 
         self.nonunique_cols = self.object_df.copy()
-        self.nonunique_cols.columns = ['A'] * len(self.nonunique_cols.columns)
+        self.nonunique_cols.columns = ["A"] * len(self.nonunique_cols.columns)
         self.nonunique_cols_nan = self.nonunique_cols.copy()
         self.nonunique_cols_nan.iloc[-1, -1] = np.nan
 
@@ -361,11 +435,10 @@ class Equals(object):
         self.object_df.equals(self.object_df_nan)
 
 
-class Interpolate(object):
+class Interpolate:
 
-    goal_time = 0.2
-    params = [None, 'infer']
-    param_names = ['downcast']
+    params = [None, "infer"]
+    param_names = ["downcast"]
 
     def setup(self, downcast):
         N = 10000
@@ -373,12 +446,16 @@ class Interpolate(object):
         self.df = DataFrame(np.random.randn(N, 100))
         self.df.values[::2] = np.nan
 
-        self.df2 = DataFrame({'A': np.arange(0, N),
-                              'B': np.random.randint(0, 100, N),
-                              'C': np.random.randn(N),
-                              'D': np.random.randn(N)})
-        self.df2.loc[1::5, 'A'] = np.nan
-        self.df2.loc[1::5, 'C'] = np.nan
+        self.df2 = DataFrame(
+            {
+                "A": np.arange(0, N),
+                "B": np.random.randint(0, 100, N),
+                "C": np.random.randn(N),
+                "D": np.random.randn(N),
+            }
+        )
+        self.df2.loc[1::5, "A"] = np.nan
+        self.df2.loc[1::5, "C"] = np.nan
 
     def time_interpolate(self, downcast):
         self.df.interpolate(downcast=downcast)
@@ -387,11 +464,10 @@ class Interpolate(object):
         self.df2.interpolate(downcast=downcast)
 
 
-class Shift(object):
+class Shift:
     # frame shift speedup issue-5609
-    goal_time = 0.2
     params = [0, 1]
-    param_names = ['axis']
+    param_names = ["axis"]
 
     def setup(self, axis):
         self.df = DataFrame(np.random.rand(10000, 500))
@@ -400,8 +476,7 @@ class Shift(object):
         self.df.shift(1, axis=axis)
 
 
-class Nunique(object):
-
+class Nunique:
     def setup(self):
         self.df = DataFrame(np.random.randn(10000, 1000))
 
@@ -409,17 +484,18 @@ class Nunique(object):
         self.df.nunique()
 
 
-class Duplicated(object):
-
-    goal_time = 0.2
-
+class Duplicated:
     def setup(self):
-        n = (1 << 20)
-        t = date_range('2015-01-01', freq='S', periods=(n // 64))
+        n = 1 << 20
+        t = date_range("2015-01-01", freq="S", periods=(n // 64))
         xs = np.random.randn(n // 64).round(2)
-        self.df = DataFrame({'a': np.random.randint(-1 << 8, 1 << 8, n),
-                             'b': np.random.choice(t, n),
-                             'c': np.random.choice(xs, n)})
+        self.df = DataFrame(
+            {
+                "a": np.random.randint(-1 << 8, 1 << 8, n),
+                "b": np.random.choice(t, n),
+                "c": np.random.choice(xs, n),
+            }
+        )
         self.df2 = DataFrame(np.random.randn(1000, 100).astype(str)).T
 
     def time_frame_duplicated(self):
@@ -429,86 +505,108 @@ class Duplicated(object):
         self.df2.duplicated()
 
 
-class XS(object):
+class XS:
 
-    goal_time = 0.2
     params = [0, 1]
-    param_names = ['axis']
+    param_names = ["axis"]
 
     def setup(self, axis):
-        self.N = 10**4
+        self.N = 10 ** 4
         self.df = DataFrame(np.random.randn(self.N, self.N))
 
     def time_frame_xs(self, axis):
         self.df.xs(self.N / 2, axis=axis)
 
 
-class SortValues(object):
+class SortValues:
 
-    goal_time = 0.2
     params = [True, False]
-    param_names = ['ascending']
+    param_names = ["ascending"]
 
     def setup(self, ascending):
-        self.df = DataFrame(np.random.randn(1000000, 2), columns=list('AB'))
+        self.df = DataFrame(np.random.randn(1000000, 2), columns=list("AB"))
 
     def time_frame_sort_values(self, ascending):
-        self.df.sort_values(by='A', ascending=ascending)
+        self.df.sort_values(by="A", ascending=ascending)
 
 
-class SortIndexByColumns(object):
-
-    goal_time = 0.2
-
+class SortIndexByColumns:
     def setup(self):
         N = 10000
         K = 10
-        self.df = DataFrame({'key1': tm.makeStringIndex(N).values.repeat(K),
-                             'key2': tm.makeStringIndex(N).values.repeat(K),
-                             'value': np.random.randn(N * K)})
+        self.df = DataFrame(
+            {
+                "key1": tm.makeStringIndex(N).values.repeat(K),
+                "key2": tm.makeStringIndex(N).values.repeat(K),
+                "value": np.random.randn(N * K),
+            }
+        )
 
     def time_frame_sort_values_by_columns(self):
-        self.df.sort_values(by=['key1', 'key2'])
+        self.df.sort_values(by=["key1", "key2"])
 
 
-class Quantile(object):
+class Quantile:
 
-    goal_time = 0.2
     params = [0, 1]
-    param_names = ['axis']
+    param_names = ["axis"]
 
     def setup(self, axis):
-        self.df = DataFrame(np.random.randn(1000, 3), columns=list('ABC'))
+        self.df = DataFrame(np.random.randn(1000, 3), columns=list("ABC"))
 
     def time_frame_quantile(self, axis):
         self.df.quantile([0.1, 0.5], axis=axis)
 
 
-class GetDtypeCounts(object):
+class GetDtypeCounts:
     # 2807
-    goal_time = 0.2
-
     def setup(self):
         self.df = DataFrame(np.random.randn(10, 10000))
 
     def time_frame_get_dtype_counts(self):
-        self.df.get_dtype_counts()
+        with warnings.catch_warnings(record=True):
+            self.df.get_dtype_counts()
 
     def time_info(self):
         self.df.info()
 
 
-class NSort(object):
+class NSort:
 
-    goal_time = 0.2
-    params = ['first', 'last']
-    param_names = ['keep']
+    params = ["first", "last", "all"]
+    param_names = ["keep"]
 
     def setup(self, keep):
-        self.df = DataFrame(np.random.randn(1000, 3), columns=list('ABC'))
+        self.df = DataFrame(np.random.randn(100000, 3), columns=list("ABC"))
 
-    def time_nlargest(self, keep):
-        self.df.nlargest(100, 'A', keep=keep)
+    def time_nlargest_one_column(self, keep):
+        self.df.nlargest(100, "A", keep=keep)
 
-    def time_nsmallest(self, keep):
-        self.df.nsmallest(100, 'A', keep=keep)
+    def time_nlargest_two_columns(self, keep):
+        self.df.nlargest(100, ["A", "B"], keep=keep)
+
+    def time_nsmallest_one_column(self, keep):
+        self.df.nsmallest(100, "A", keep=keep)
+
+    def time_nsmallest_two_columns(self, keep):
+        self.df.nsmallest(100, ["A", "B"], keep=keep)
+
+
+class Describe:
+    def setup(self):
+        self.df = DataFrame(
+            {
+                "a": np.random.randint(0, 100, int(1e6)),
+                "b": np.random.randint(0, 100, int(1e6)),
+                "c": np.random.randint(0, 100, int(1e6)),
+            }
+        )
+
+    def time_series_describe(self):
+        self.df["a"].describe()
+
+    def time_dataframe_describe(self):
+        self.df.describe()
+
+
+from .pandas_vb_common import setup  # noqa: F401
