@@ -1,25 +1,26 @@
 import datetime
 
-from pandas import Timestamp
+import dateutil
 import pytz
 
+from pandas import Timestamp
 
-class TimestampConstruction(object):
 
+class TimestampConstruction:
     def time_parse_iso8601_no_tz(self):
-        Timestamp('2017-08-25 08:16:14')
+        Timestamp("2017-08-25 08:16:14")
 
     def time_parse_iso8601_tz(self):
-        Timestamp('2017-08-25 08:16:14-0500')
+        Timestamp("2017-08-25 08:16:14-0500")
 
     def time_parse_dateutil(self):
-        Timestamp('2017/08/25 08:16:14 AM')
+        Timestamp("2017/08/25 08:16:14 AM")
 
     def time_parse_today(self):
-        Timestamp('today')
+        Timestamp("today")
 
     def time_parse_now(self):
-        Timestamp('now')
+        Timestamp("now")
 
     def time_fromordinal(self):
         Timestamp.fromordinal(730120)
@@ -28,16 +29,14 @@ class TimestampConstruction(object):
         Timestamp.fromtimestamp(1515448538)
 
 
-class TimestampProperties(object):
-    goal_time = 0.2
-
-    _tzs = [None, pytz.timezone('Europe/Amsterdam')]
-    _freqs = [None, 'B']
+class TimestampProperties:
+    _tzs = [None, pytz.timezone("Europe/Amsterdam"), pytz.UTC, dateutil.tz.tzutc()]
+    _freqs = [None, "B"]
     params = [_tzs, _freqs]
-    param_names = ['tz', 'freq']
+    param_names = ["tz", "freq"]
 
     def setup(self, tz, freq):
-        self.ts = Timestamp('2017-08-25 08:16:14', tzinfo=tz, freq=freq)
+        self.ts = Timestamp("2017-08-25 08:16:14", tzinfo=tz, freq=freq)
 
     def time_tz(self, tz, freq):
         self.ts.tz
@@ -46,7 +45,7 @@ class TimestampProperties(object):
         self.ts.dayofweek
 
     def time_weekday_name(self, tz, freq):
-        self.ts.weekday_name
+        self.ts.day_name
 
     def time_dayofyear(self, tz, freq):
         self.ts.dayofyear
@@ -76,29 +75,30 @@ class TimestampProperties(object):
         self.ts.is_quarter_end
 
     def time_is_year_start(self, tz, freq):
-        self.ts.is_quarter_end
+        self.ts.is_year_start
 
     def time_is_year_end(self, tz, freq):
-        self.ts.is_quarter_end
+        self.ts.is_year_end
 
     def time_is_leap_year(self, tz, freq):
-        self.ts.is_quarter_end
+        self.ts.is_leap_year
 
     def time_microsecond(self, tz, freq):
         self.ts.microsecond
 
+    def time_month_name(self, tz, freq):
+        self.ts.month_name()
 
-class TimestampOps(object):
-    goal_time = 0.2
 
-    params = [None, 'US/Eastern']
-    param_names = ['tz']
+class TimestampOps:
+    params = [None, "US/Eastern", pytz.UTC, dateutil.tz.tzutc()]
+    param_names = ["tz"]
 
     def setup(self, tz):
-        self.ts = Timestamp('2017-08-25 08:16:14', tz=tz)
+        self.ts = Timestamp("2017-08-25 08:16:14", tz=tz)
 
     def time_replace_tz(self, tz):
-        self.ts.replace(tzinfo=pytz.timezone('US/Eastern'))
+        self.ts.replace(tzinfo=pytz.timezone("US/Eastern"))
 
     def time_replace_None(self, tz):
         self.ts.replace(tzinfo=None)
@@ -106,13 +106,31 @@ class TimestampOps(object):
     def time_to_pydatetime(self, tz):
         self.ts.to_pydatetime()
 
+    def time_normalize(self, tz):
+        self.ts.normalize()
 
-class TimestampAcrossDst(object):
-    goal_time = 0.2
+    def time_tz_convert(self, tz):
+        if self.ts.tz is not None:
+            self.ts.tz_convert(tz)
 
+    def time_tz_localize(self, tz):
+        if self.ts.tz is None:
+            self.ts.tz_localize(tz)
+
+    def time_to_julian_date(self, tz):
+        self.ts.to_julian_date()
+
+    def time_floor(self, tz):
+        self.ts.floor("5T")
+
+    def time_ceil(self, tz):
+        self.ts.ceil("5T")
+
+
+class TimestampAcrossDst:
     def setup(self):
         dt = datetime.datetime(2016, 3, 27, 1)
-        self.tzinfo = pytz.timezone('CET').localize(dt, is_dst=False).tzinfo
+        self.tzinfo = pytz.timezone("CET").localize(dt, is_dst=False).tzinfo
         self.ts2 = Timestamp(dt)
 
     def time_replace_across_dst(self):
