@@ -17,37 +17,36 @@ from announce import build_components
 
 class ContributorsDirective(Directive):
     required_arguments = 1
-    name = 'contributors'
+    name = "contributors"
 
     def run(self):
         range_ = self.arguments[0]
+        if range_.endswith("x..HEAD"):
+            return [nodes.paragraph(), nodes.bullet_list()]
         try:
             components = build_components(range_)
-        except git.GitCommandError:
+        except git.GitCommandError as exc:
             return [
                 self.state.document.reporter.warning(
-                    "Cannot find contributors for range '{}'".format(range_),
-                    line=self.lineno)
+                    "Cannot find contributors for range '{}': {}".format(range_, exc),
+                    line=self.lineno,
+                )
             ]
         else:
             message = nodes.paragraph()
-            message += nodes.Text(components['author_message'])
+            message += nodes.Text(components["author_message"])
 
             listnode = nodes.bullet_list()
 
-            for author in components['authors']:
+            for author in components["authors"]:
                 para = nodes.paragraph()
                 para += nodes.Text(author)
-                listnode += nodes.list_item('', para)
+                listnode += nodes.list_item("", para)
 
         return [message, listnode]
 
 
 def setup(app):
-    app.add_directive('contributors', ContributorsDirective)
+    app.add_directive("contributors", ContributorsDirective)
 
-    return {
-        'version': '0.1',
-        'parallel_read_safe': True,
-        'parallel_write_safe': True,
-    }
+    return {"version": "0.1", "parallel_read_safe": True, "parallel_write_safe": True}
