@@ -35,14 +35,12 @@ http://www.opensource.apple.com/source/tcl/tcl-14/tcl/license.terms
  * Copyright (c) 1994 Sun Microsystems, Inc.
 */
 
-// "py_defines.h" needs to be included first to
-// avoid compilation errors, but it does violate
-// styleguide checks with regards to include order.
-#include "py_defines.h"
 #define PY_ARRAY_UNIQUE_SYMBOL UJSON_NUMPY
 #define NO_IMPORT_ARRAY
-#include <numpy/arrayobject.h>  // NOLINT(build/include_order)
-#include <ultrajson.h>          // NOLINT(build/include_order)
+#define PY_SSIZE_T_CLEAN
+#include <Python.h>
+#include <numpy/arrayobject.h>
+#include <ultrajson.h>
 
 #define PRINTMARK()
 
@@ -409,7 +407,7 @@ JSOBJ Object_npyEndObject(void *prv, JSOBJ obj) {
 }
 
 int Object_npyObjectAddKey(void *prv, JSOBJ obj, JSOBJ name, JSOBJ value) {
-    PyObject *label;
+    PyObject *label, *labels;
     npy_intp labelidx;
     // add key to label array, value to values array
     NpyArrContext *npyarr = (NpyArrContext *)obj;
@@ -424,11 +422,11 @@ int Object_npyObjectAddKey(void *prv, JSOBJ obj, JSOBJ name, JSOBJ value) {
     if (!npyarr->labels[labelidx]) {
         npyarr->labels[labelidx] = PyList_New(0);
     }
-
+    labels = npyarr->labels[labelidx];
     // only fill label array once, assumes all column labels are the same
     // for 2-dimensional arrays.
-    if (PyList_GET_SIZE(npyarr->labels[labelidx]) <= npyarr->elcount) {
-        PyList_Append(npyarr->labels[labelidx], label);
+    if (PyList_Check(labels) && PyList_GET_SIZE(labels) <= npyarr->elcount) {
+        PyList_Append(labels, label);
     }
 
     if (((JSONObjectDecoder *)npyarr->dec)->arrayAddItem(prv, obj, value)) {
@@ -439,16 +437,16 @@ int Object_npyObjectAddKey(void *prv, JSOBJ obj, JSOBJ name, JSOBJ value) {
 }
 
 int Object_objectAddKey(void *prv, JSOBJ obj, JSOBJ name, JSOBJ value) {
-    PyDict_SetItem(obj, name, value);
+    int ret = PyDict_SetItem(obj, name, value);
     Py_DECREF((PyObject *)name);
     Py_DECREF((PyObject *)value);
-    return 1;
+    return ret == 0 ? 1 : 0;
 }
 
 int Object_arrayAddItem(void *prv, JSOBJ obj, JSOBJ value) {
-    PyList_Append(obj, value);
+    int ret = PyList_Append(obj, value);
     Py_DECREF((PyObject *)value);
-    return 1;
+    return ret == 0 ? 1 : 0;
 }
 
 JSOBJ Object_newString(void *prv, wchar_t *start, wchar_t *end) {
@@ -470,7 +468,7 @@ JSOBJ Object_newArray(void *prv, void *decoder) { return PyList_New(0); }
 JSOBJ Object_endArray(void *prv, JSOBJ obj) { return obj; }
 
 JSOBJ Object_newInteger(void *prv, JSINT32 value) {
-    return PyInt_FromLong((long)value);
+    return PyLong_FromLong((long)value);
 }
 
 JSOBJ Object_newLong(void *prv, JSINT64 value) {
@@ -530,7 +528,7 @@ PyObject *JSONToObj(PyObject *self, PyObject *args, PyObject *kwargs) {
         decoder->preciseFloat = 1;
     }
 
-    if (PyString_Check(arg)) {
+    if (PyBytes_Check(arg)) {
         sarg = arg;
     } else if (PyUnicode_Check(arg)) {
         sarg = PyUnicode_AsUTF8String(arg);
@@ -539,7 +537,7 @@ PyObject *JSONToObj(PyObject *self, PyObject *args, PyObject *kwargs) {
             return NULL;
         }
     } else {
-        PyErr_Format(PyExc_TypeError, "Expected String or Unicode");
+        PyErr_Format(PyExc_TypeError, "Expected 'str' or 'bytes'");
         return NULL;
     }
 
@@ -559,8 +557,8 @@ PyObject *JSONToObj(PyObject *self, PyObject *args, PyObject *kwargs) {
         }
     }
 
-    ret = JSON_DecodeObject(decoder, PyString_AS_STRING(sarg),
-                            PyString_GET_SIZE(sarg));
+    ret = JSON_DecodeObject(decoder, PyBytes_AS_STRING(sarg),
+                            PyBytes_GET_SIZE(sarg));
 
     if (sarg != arg) {
         Py_DECREF(sarg);
@@ -591,48 +589,3 @@ PyObject *JSONToObj(PyObject *self, PyObject *args, PyObject *kwargs) {
 
     return ret;
 }
-
-PyObject *JSONFileToObj(PyObject *self, PyObject *args, PyObject *kwargs) {
-    PyObject *read;
-    PyObject *string;
-    PyObject *result;
-    PyObject *file = NULL;
-    PyObject *argtuple;
-
-    if (!PyArg_ParseTuple(args, "O", &file)) {
-        return NULL;
-    }
-
-    if (!PyObject_HasAttrString(file, "read")) {
-        PyErr_Format(PyExc_TypeError, "expected file");
-        return NULL;
-    }
-
-    read = PyObject_GetAttrString(file, "read");
-
-    if (!PyCallable_Check(read)) {
-        Py_XDECREF(read);
-        PyErr_Format(PyExc_TypeError, "expected file");
-        return NULL;
-    }
-
-    string = PyObject_CallObject(read, NULL);
-    Py_XDECREF(read);
-
-    if (string == NULL) {
-        return NULL;
-    }
-
-    argtuple = PyTuple_Pack(1, string);
-
-    result = JSONToObj(self, argtuple, kwargs);
-
-    Py_XDECREF(argtuple);
-    Py_XDECREF(string);
-
-    if (result == NULL) {
-        return NULL;
-    }
-
-    return result;
-}
