@@ -35,8 +35,9 @@ http://www.opensource.apple.com/source/tcl/tcl-14/tcl/license.terms
 * Copyright (c) 1994 Sun Microsystems, Inc.
 */
 
-#include "py_defines.h"
 #include "version.h"
+#define PY_SSIZE_T_CLEAN
+#include <Python.h>
 
 /* objToJSON */
 PyObject *objToJSON(PyObject *self, PyObject *args, PyObject *kwargs);
@@ -45,12 +46,6 @@ void initObjToJSON(void);
 /* JSONToObj */
 PyObject *JSONToObj(PyObject *self, PyObject *args, PyObject *kwargs);
 
-/* objToJSONFile */
-PyObject *objToJSONFile(PyObject *self, PyObject *args, PyObject *kwargs);
-
-/* JSONFileToObj */
-PyObject *JSONFileToObj(PyObject *self, PyObject *args, PyObject *kwargs);
-
 #define ENCODER_HELP_TEXT                                                  \
     "Use ensure_ascii=false to output UTF-8. Pass in double_precision to " \
     "alter the maximum digit precision of doubles. Set "                   \
@@ -67,17 +62,9 @@ static PyMethodDef ujsonMethods[] = {
     {"loads", (PyCFunction)JSONToObj, METH_VARARGS | METH_KEYWORDS,
      "Converts JSON as string to dict object structure. Use precise_float=True "
      "to use high precision float decoder."},
-    {"dump", (PyCFunction)objToJSONFile, METH_VARARGS | METH_KEYWORDS,
-     "Converts arbitrary object recursively into JSON "
-     "file. " ENCODER_HELP_TEXT},
-    {"load", (PyCFunction)JSONFileToObj, METH_VARARGS | METH_KEYWORDS,
-     "Converts JSON as file to dict object structure. Use precise_float=True "
-     "to use high precision float decoder."},
     {NULL, NULL, 0, NULL} /* Sentinel */
 };
 
-#if PY_MAJOR_VERSION >= 3
-
 static struct PyModuleDef moduledef = {
     PyModuleDef_HEAD_INIT,
     "_libjson",
@@ -94,14 +81,6 @@ static struct PyModuleDef moduledef = {
 #define PYMODULE_CREATE() PyModule_Create(&moduledef)
 #define MODINITERROR return NULL
 
-#else
-
-#define PYMODINITFUNC PyMODINIT_FUNC initjson(void)
-#define PYMODULE_CREATE() Py_InitModule("json", ujsonMethods)
-#define MODINITERROR return
-
-#endif
-
 PYMODINITFUNC {
     PyObject *module;
     PyObject *version_string;
@@ -113,10 +92,8 @@ PYMODINITFUNC {
         MODINITERROR;
     }
 
-    version_string = PyString_FromString(UJSON_VERSION);
+    version_string = PyUnicode_FromString(UJSON_VERSION);
     PyModule_AddObject(module, "__version__", version_string);
 
-#if PY_MAJOR_VERSION >= 3
     return module;
-#endif
 }
