@@ -1,51 +1,89 @@
 import operator
+from typing import Any
+import warnings
 
 import numpy as np
+
+from pandas._config import get_option
+
 from pandas._libs import index as libindex
-
-from pandas import compat
+import pandas.compat as compat
 from pandas.compat.numpy import function as nv
-from pandas.core.dtypes.generic import ABCCategorical, ABCSeries
-from pandas.core.dtypes.dtypes import CategoricalDtype
-from pandas.core.dtypes.common import (
-    is_categorical_dtype,
-    _ensure_platform_int,
-    is_list_like,
-    is_interval_dtype,
-    is_scalar)
-from pandas.core.dtypes.missing import array_equivalent, isna
-from pandas.core.algorithms import take_1d
-
-
 from pandas.util._decorators import Appender, cache_readonly
-from pandas.core.config import get_option
-from pandas.core.indexes.base import Index, _index_shared_docs
+
+from pandas.core.dtypes.common import (
+    ensure_platform_int,
+    is_categorical_dtype,
+    is_interval_dtype,
+    is_list_like,
+    is_scalar,
+)
+from pandas.core.dtypes.dtypes import CategoricalDtype
+from pandas.core.dtypes.generic import ABCCategorical, ABCSeries
+from pandas.core.dtypes.missing import isna
+
+from pandas._typing import AnyArrayLike
 from pandas.core import accessor
+from pandas.core.algorithms import take_1d
+from pandas.core.arrays.categorical import Categorical, contains
 import pandas.core.common as com
-import pandas.core.missing as missing
 import pandas.core.indexes.base as ibase
+from pandas.core.indexes.base import Index, _index_shared_docs
+import pandas.core.missing as missing
+from pandas.core.ops import get_op_result_name
 
 _index_doc_kwargs = dict(ibase._index_doc_kwargs)
-_index_doc_kwargs.update(dict(target_klass='CategoricalIndex'))
+_index_doc_kwargs.update(dict(target_klass="CategoricalIndex"))
 
 
+@accessor.delegate_names(
+    delegate=Categorical,
+    accessors=[
+        "rename_categories",
+        "reorder_categories",
+        "add_categories",
+        "remove_categories",
+        "remove_unused_categories",
+        "set_categories",
+        "as_ordered",
+        "as_unordered",
+        "min",
+        "max",
+    ],
+    typ="method",
+    overwrite=True,
+)
 class CategoricalIndex(Index, accessor.PandasDelegate):
     """
+    Index based on an underlying :class:`Categorical`.
 
-    Immutable Index implementing an ordered, sliceable set. CategoricalIndex
-    represents a sparsely populated Index with an underlying Categorical.
+    CategoricalIndex, like Categorical, can only take on a limited,
+    and usually fixed, number of possible values (`categories`). Also,
+    like Categorical, it might have an order, but numerical operations
+    (additions, divisions, ...) are not possible.
 
     Parameters
     ----------
-    data : array-like or Categorical, (1-dimensional)
-    categories : optional, array-like
-        categories for the CategoricalIndex
-    ordered : boolean,
-        designating if the categories are ordered
-    copy : bool
-        Make a copy of input ndarray
-    name : object
-        Name to be stored in the index
+    data : array-like (1-dimensional)
+        The values of the categorical. If `categories` are given, values not in
+        `categories` will be replaced with NaN.
+    categories : index-like, optional
+        The categories for the categorical. Items need to be unique.
+        If the categories are not given here (and also not in `dtype`), they
+        will be inferred from the `data`.
+    ordered : bool, optional
+        Whether or not this categorical is treated as an ordered
+        categorical. If not given here or in `dtype`, the resulting
+        categorical will be unordered.
+    dtype : CategoricalDtype or the string "category", optional
+        If :class:`CategoricalDtype`, cannot be used together with
+        `categories` or `ordered`.
+
+        .. versionadded:: 0.21.0
+    copy : bool, default False
+        Make a copy of input ndarray.
+    name : object, optional
+        Name to be stored in the index.
 
     Attributes
     ----------
@@ -65,49 +103,106 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
     as_unordered
     map
 
+    Raises
+    ------
+    ValueError
+        If the categories do not validate.
+    TypeError
+        If an explicit ``ordered=True`` is given but no `categories` and the
+        `values` are not sortable.
+
     See Also
     --------
-    Categorical, Index
+    Index : The base pandas Index type.
+    Categorical : A categorical array.
+    CategoricalDtype : Type for categorical data.
+
+    Notes
+    -----
+    See the `user guide
+    <http://pandas.pydata.org/pandas-docs/stable/user_guide/advanced.html#categoricalindex>`_
+    for more.
+
+    Examples
+    --------
+    >>> pd.CategoricalIndex(['a', 'b', 'c', 'a', 'b', 'c'])
+    CategoricalIndex(['a', 'b', 'c', 'a', 'b', 'c'], categories=['a', 'b', 'c'], ordered=False, dtype='category')  # noqa
+
+    ``CategoricalIndex`` can also be instantiated from a ``Categorical``:
+
+    >>> c = pd.Categorical(['a', 'b', 'c', 'a', 'b', 'c'])
+    >>> pd.CategoricalIndex(c)
+    CategoricalIndex(['a', 'b', 'c', 'a', 'b', 'c'], categories=['a', 'b', 'c'], ordered=False, dtype='category')  # noqa
+
+    Ordered ``CategoricalIndex`` can have a min and max value.
+
+    >>> ci = pd.CategoricalIndex(['a','b','c','a','b','c'], ordered=True,
+    ...                          categories=['c', 'b', 'a'])
+    >>> ci
+    CategoricalIndex(['a', 'b', 'c', 'a', 'b', 'c'], categories=['c', 'b', 'a'], ordered=True, dtype='category')  # noqa
+    >>> ci.min()
+    'c'
     """
 
-    _typ = 'categoricalindex'
-    _engine_type = libindex.Int64Engine
-    _attributes = ['name']
+    _typ = "categoricalindex"
 
-    def __new__(cls, data=None, categories=None, ordered=None, dtype=None,
-                copy=False, name=None, fastpath=False):
+    @property
+    def _engine_type(self):
+        # self.codes can have dtype int8, int16, int32 or int64, so we need
+        # to return the corresponding engine type (libindex.Int8Engine, etc.).
+        return {
+            np.int8: libindex.Int8Engine,
+            np.int16: libindex.Int16Engine,
+            np.int32: libindex.Int32Engine,
+            np.int64: libindex.Int64Engine,
+        }[self.codes.dtype.type]
 
-        if fastpath:
-            return cls._simple_new(data, name=name, dtype=dtype)
+    _attributes = ["name"]
 
-        if name is None and hasattr(data, 'name'):
+    # --------------------------------------------------------------------
+    # Constructors
+
+    def __new__(
+        cls,
+        data=None,
+        categories=None,
+        ordered=None,
+        dtype=None,
+        copy=False,
+        name=None,
+        fastpath=None,
+    ):
+
+        if fastpath is not None:
+            warnings.warn(
+                "The 'fastpath' keyword is deprecated, and will be "
+                "removed in a future version.",
+                FutureWarning,
+                stacklevel=2,
+            )
+            if fastpath:
+                return cls._simple_new(data, name=name, dtype=dtype)
+
+        dtype = CategoricalDtype._from_values_or_dtype(data, categories, ordered, dtype)
+
+        if name is None and hasattr(data, "name"):
             name = data.name
 
-        if isinstance(data, ABCCategorical):
-            data = cls._create_categorical(cls, data, categories, ordered,
-                                           dtype)
-        elif isinstance(data, CategoricalIndex):
-            data = data._data
-            data = cls._create_categorical(cls, data, categories, ordered,
-                                           dtype)
-        else:
-
+        if not is_categorical_dtype(data):
             # don't allow scalars
             # if data is None, then categories must be provided
             if is_scalar(data):
                 if data is not None or categories is None:
                     cls._scalar_data_error(data)
                 data = []
-            data = cls._create_categorical(cls, data, categories, ordered,
-                                           dtype)
 
-        if copy:
-            data = data.copy()
+        data = cls._create_categorical(data, dtype=dtype)
+
+        data = data.copy() if copy else data
 
         return cls._simple_new(data, name=name)
 
-    def _create_from_codes(self, codes, categories=None, ordered=None,
-                           name=None):
+    def _create_from_codes(self, codes, dtype=None, name=None):
         """
         *this is an internal non-public method*
 
@@ -116,8 +211,7 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
         Parameters
         ----------
         codes : new codes
-        categories : optional categories, defaults to existing
-        ordered : optional ordered attribute, defaults to existing
+        dtype: CategoricalDtype, defaults to existing
         name : optional name attribute, defaults to existing
 
         Returns
@@ -125,20 +219,15 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
         CategoricalIndex
         """
 
-        from pandas.core.arrays import Categorical
-        if categories is None:
-            categories = self.categories
-        if ordered is None:
-            ordered = self.ordered
+        if dtype is None:
+            dtype = self.dtype
         if name is None:
             name = self.name
-        cat = Categorical.from_codes(codes, categories=categories,
-                                     ordered=self.ordered)
+        cat = Categorical.from_codes(codes, dtype=dtype)
         return CategoricalIndex(cat, name=name)
 
-    @staticmethod
-    def _create_categorical(self, data, categories=None, ordered=None,
-                            dtype=None):
+    @classmethod
+    def _create_categorical(cls, data, dtype=None):
         """
         *this is an internal non-public method*
 
@@ -147,73 +236,43 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
         Parameters
         ----------
         data : data for new Categorical
-        categories : optional categories, defaults to existing
-        ordered : optional ordered attribute, defaults to existing
         dtype : CategoricalDtype, defaults to existing
 
         Returns
         -------
         Categorical
         """
-        if (isinstance(data, (ABCSeries, type(self))) and
-                is_categorical_dtype(data)):
+        if isinstance(data, (cls, ABCSeries)) and is_categorical_dtype(data):
             data = data.values
 
         if not isinstance(data, ABCCategorical):
-            if ordered is None and dtype is None:
-                ordered = False
-            from pandas.core.arrays import Categorical
-            data = Categorical(data, categories=categories, ordered=ordered,
-                               dtype=dtype)
-        else:
-            if categories is not None:
-                data = data.set_categories(categories, ordered=ordered)
-            elif ordered is not None and ordered != data.ordered:
-                data = data.set_ordered(ordered)
-            if isinstance(dtype, CategoricalDtype):
-                # we want to silently ignore dtype='category'
-                data = data._set_dtype(dtype)
+            return Categorical(data, dtype=dtype)
+
+        if isinstance(dtype, CategoricalDtype) and dtype != data.dtype:
+            # we want to silently ignore dtype='category'
+            data = data._set_dtype(dtype)
         return data
 
     @classmethod
-    def _simple_new(cls, values, name=None, categories=None, ordered=None,
-                    dtype=None, **kwargs):
+    def _simple_new(cls, values, name=None, dtype=None, **kwargs):
         result = object.__new__(cls)
 
-        values = cls._create_categorical(cls, values, categories, ordered,
-                                         dtype=dtype)
+        values = cls._create_categorical(values, dtype=dtype)
         result._data = values
         result.name = name
-        for k, v in compat.iteritems(kwargs):
+        for k, v in kwargs.items():
             setattr(result, k, v)
 
         result._reset_identity()
         return result
 
-    @Appender(_index_shared_docs['_shallow_copy'])
-    def _shallow_copy(self, values=None, categories=None, ordered=None,
-                      dtype=None, **kwargs):
-        # categories and ordered can't be part of attributes,
-        # as these are properties
-        # we want to reuse self.dtype if possible, i.e. neither are
-        # overridden.
-        if dtype is not None and (categories is not None or
-                                  ordered is not None):
-            raise TypeError("Cannot specify both `dtype` and `categories` "
-                            "or `ordered`")
+    # --------------------------------------------------------------------
 
-        if categories is None and ordered is None:
-            dtype = self.dtype if dtype is None else dtype
-            return super(CategoricalIndex, self)._shallow_copy(
-                values=values, dtype=dtype, **kwargs)
-        if categories is None:
-            categories = self.categories
-        if ordered is None:
-            ordered = self.ordered
-
-        return super(CategoricalIndex, self)._shallow_copy(
-            values=values, categories=categories,
-            ordered=ordered, **kwargs)
+    @Appender(_index_shared_docs["_shallow_copy"])
+    def _shallow_copy(self, values=None, dtype=None, **kwargs):
+        if dtype is None:
+            dtype = self.dtype
+        return super()._shallow_copy(values=values, dtype=dtype, **kwargs)
 
     def _is_dtype_compat(self, other):
         """
@@ -230,23 +289,30 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
             if isinstance(other, CategoricalIndex):
                 other = other._values
             if not other.is_dtype_equal(self):
-                raise TypeError("categories must match existing categories "
-                                "when appending")
+                raise TypeError(
+                    "categories must match existing categories " "when appending"
+                )
         else:
             values = other
             if not is_list_like(values):
                 values = [values]
-            other = CategoricalIndex(self._create_categorical(
-                self, other, categories=self.categories, ordered=self.ordered))
+            other = CategoricalIndex(self._create_categorical(other, dtype=self.dtype))
             if not other.isin(values).all():
-                raise TypeError("cannot append a non-category item to a "
-                                "CategoricalIndex")
+                raise TypeError(
+                    "cannot append a non-category item to a " "CategoricalIndex"
+                )
 
         return other
 
     def equals(self, other):
         """
-        Determines if two CategorialIndex objects contain the same elements.
+        Determine if two CategoricalIndex objects contain the same elements.
+
+        Returns
+        -------
+        bool
+            If two CategoricalIndex objects have equal elements True,
+            otherwise False.
         """
         if self.is_(other):
             return True
@@ -256,12 +322,17 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
 
         try:
             other = self._is_dtype_compat(other)
-            return array_equivalent(self._data, other)
+            if isinstance(other, type(self)):
+                other = other._data
+            return self._data.equals(other)
         except (TypeError, ValueError):
             pass
 
         return False
 
+    # --------------------------------------------------------------------
+    # Rendering Methods
+
     @property
     def _formatter_func(self):
         return self.categories._formatter_func
@@ -270,24 +341,31 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
         """
         Return a list of tuples of the (attr,formatted_value)
         """
-        max_categories = (10 if get_option("display.max_categories") == 0 else
-                          get_option("display.max_categories"))
+        max_categories = (
+            10
+            if get_option("display.max_categories") == 0
+            else get_option("display.max_categories")
+        )
         attrs = [
-            ('categories',
-             ibase.default_pprint(self.categories,
-                                  max_seq_items=max_categories)),
-            ('ordered', self.ordered)]
+            (
+                "categories",
+                ibase.default_pprint(self.categories, max_seq_items=max_categories),
+            ),
+            ("ordered", self.ordered),
+        ]
         if self.name is not None:
-            attrs.append(('name', ibase.default_pprint(self.name)))
-        attrs.append(('dtype', "'%s'" % self.dtype.name))
-        max_seq_items = get_option('display.max_seq_items') or len(self)
+            attrs.append(("name", ibase.default_pprint(self.name)))
+        attrs.append(("dtype", "'%s'" % self.dtype.name))
+        max_seq_items = get_option("display.max_seq_items") or len(self)
         if len(self) > max_seq_items:
-            attrs.append(('length', len(self)))
+            attrs.append(("length", len(self)))
         return attrs
 
+    # --------------------------------------------------------------------
+
     @property
     def inferred_type(self):
-        return 'categorical'
+        return "categorical"
 
     @property
     def values(self):
@@ -299,9 +377,14 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
         # Size of the items in categories, not codes.
         return self.values.itemsize
 
-    def get_values(self):
-        """ return the underlying data as an ndarray """
-        return self._data.get_values()
+    def _wrap_setop_result(self, other, result):
+        name = get_op_result_name(self, other)
+        return self._shallow_copy(result, name=name)
+
+    def _internal_get_values(self):
+        # override base Index version to get the numpy array representation of
+        # the underlying Categorical
+        return self._data._internal_get_values()
 
     def tolist(self):
         return self._data.tolist()
@@ -321,32 +404,23 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
     def _reverse_indexer(self):
         return self._data._reverse_indexer()
 
-    @Appender(_index_shared_docs['__contains__'] % _index_doc_kwargs)
+    @Appender(_index_shared_docs["contains"] % _index_doc_kwargs)
     def __contains__(self, key):
-        hash(key)
+        # if key is a NaN, check if any NaN is in self.
+        if is_scalar(key) and isna(key):
+            return self.hasnans
 
-        if self.categories._defer_to_indexing:
-            return key in self.categories
-
-        return key in self.values
-
-    @Appender(_index_shared_docs['contains'] % _index_doc_kwargs)
-    def contains(self, key):
-        hash(key)
-
-        if self.categories._defer_to_indexing:
-            return self.categories.contains(key)
-
-        return key in self.values
+        return contains(self, key, container=self._engine)
 
     def __array__(self, dtype=None):
         """ the array interface, return my values """
         return np.array(self._data, dtype=dtype)
 
-    @Appender(_index_shared_docs['astype'])
+    @Appender(_index_shared_docs["astype"])
     def astype(self, dtype, copy=True):
         if is_interval_dtype(dtype):
             from pandas import IntervalIndex
+
             return IntervalIndex(np.array(self))
         elif is_categorical_dtype(dtype):
             # GH 18630
@@ -354,14 +428,14 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
             if dtype == self.dtype:
                 return self.copy() if copy else self
 
-        return super(CategoricalIndex, self).astype(dtype=dtype, copy=copy)
+        return super().astype(dtype=dtype, copy=copy)
 
     @cache_readonly
     def _isnan(self):
         """ return if each value is nan"""
         return self._data.codes == -1
 
-    @Appender(ibase._index_shared_docs['fillna'])
+    @Appender(ibase._index_shared_docs["fillna"])
     def fillna(self, value, downcast=None):
         self._assert_can_do_op(value)
         return CategoricalIndex(self._data.fillna(value), name=self.name)
@@ -371,42 +445,44 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
 
     @cache_readonly
     def _engine(self):
-
-        # we are going to look things up with the codes themselves
-        return self._engine_type(lambda: self.codes.astype('i8'), len(self))
+        # we are going to look things up with the codes themselves.
+        # To avoid a reference cycle, bind `codes` to a local variable, so
+        # `self` is not passed into the lambda.
+        codes = self.codes
+        return self._engine_type(lambda: codes, len(self))
 
     # introspection
     @cache_readonly
     def is_unique(self):
-        return not self.duplicated().any()
+        return self._engine.is_unique
 
     @property
     def is_monotonic_increasing(self):
-        return Index(self.codes).is_monotonic_increasing
+        return self._engine.is_monotonic_increasing
 
     @property
     def is_monotonic_decreasing(self):
-        return Index(self.codes).is_monotonic_decreasing
+        return self._engine.is_monotonic_decreasing
 
-    @Appender(_index_shared_docs['index_unique'] % _index_doc_kwargs)
+    @Appender(_index_shared_docs["index_unique"] % _index_doc_kwargs)
     def unique(self, level=None):
         if level is not None:
             self._validate_index_level(level)
         result = self.values.unique()
-        # CategoricalIndex._shallow_copy keeps original categories
-        # and ordered if not otherwise specified
-        return self._shallow_copy(result, categories=result.categories,
-                                  ordered=result.ordered)
+        # CategoricalIndex._shallow_copy keeps original dtype
+        # if not otherwise specified
+        return self._shallow_copy(result, dtype=result.dtype)
 
     @Appender(Index.duplicated.__doc__)
-    def duplicated(self, keep='first'):
+    def duplicated(self, keep="first"):
         from pandas._libs.hashtable import duplicated_int64
-        codes = self.codes.astype('i8')
+
+        codes = self.codes.astype("i8")
         return duplicated_int64(codes, keep)
 
     def _to_safe_for_reshape(self):
         """ convert to object if we are a categorical """
-        return self.astype('object')
+        return self.astype("object")
 
     def get_loc(self, key, method=None):
         """
@@ -422,8 +498,12 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
         -------
         loc : int if unique index, slice if monotonic index, else mask
 
+        Raises
+        ------
+        KeyError : if the key is not in the index
+
         Examples
-        ---------
+        --------
         >>> unique_index = pd.CategoricalIndex(list('abc'))
         >>> unique_index.get_loc('b')
         1
@@ -432,49 +512,63 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
         >>> monotonic_index.get_loc('b')
         slice(1, 3, None)
 
-        >>> non_monotonic_index = p.dCategoricalIndex(list('abcb'))
+        >>> non_monotonic_index = pd.CategoricalIndex(list('abcb'))
         >>> non_monotonic_index.get_loc('b')
         array([False,  True, False,  True], dtype=bool)
         """
-        codes = self.categories.get_loc(key)
-        if (codes == -1):
+        code = self.categories.get_loc(key)
+        code = self.codes.dtype.type(code)
+        try:
+            return self._engine.get_loc(code)
+        except KeyError:
             raise KeyError(key)
-        return self._engine.get_loc(codes)
 
-    def get_value(self, series, key):
+    def get_value(self, series: AnyArrayLike, key: Any):
         """
         Fast lookup of value from 1-dimensional ndarray. Only use this if you
         know what you're doing
+
+        Parameters
+        ----------
+        series : Series, ExtensionArray, Index, or ndarray
+            1-dimensional array to take values from
+        key: : scalar
+            The value of this index at the position of the desired value,
+            otherwise the positional index of the desired value
+
+        Returns
+        -------
+        Any
+            The element of the series at the position indicated by the key
         """
         try:
-            k = com._values_from_object(key)
-            k = self._convert_scalar_indexer(k, kind='getitem')
+            k = com.values_from_object(key)
+            k = self._convert_scalar_indexer(k, kind="getitem")
             indexer = self.get_loc(k)
-            return series.iloc[indexer]
+            return series.take([indexer])[0]
         except (KeyError, TypeError):
             pass
 
         # we might be a positional inexer
-        return super(CategoricalIndex, self).get_value(series, key)
+        return super().get_value(series, key)
 
     def _can_reindex(self, indexer):
         """ always allow reindexing """
         pass
 
-    @Appender(_index_shared_docs['where'])
+    @Appender(_index_shared_docs["where"])
     def where(self, cond, other=None):
+        # TODO: Investigate an alternative implementation with
+        # 1. copy the underlying Categorical
+        # 2. setitem with `cond` and `other`
+        # 3. Rebuild CategoricalIndex.
         if other is None:
             other = self._na_value
         values = np.where(cond, self.values, other)
-
-        from pandas.core.arrays import Categorical
-        cat = Categorical(values,
-                          categories=self.categories,
-                          ordered=self.ordered)
+        cat = Categorical(values, dtype=self.dtype)
         return self._shallow_copy(cat, **self._get_attributes_dict())
 
-    def reindex(self, target, method=None, level=None, limit=None,
-                tolerance=None):
+    def reindex(self, target, method=None, level=None, limit=None, tolerance=None):
         """
         Create index with target's values (move/add/delete values as necessary)
 
@@ -488,23 +582,30 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
         """
 
         if method is not None:
-            raise NotImplementedError("argument method is not implemented for "
-                                      "CategoricalIndex.reindex")
+            raise NotImplementedError(
+                "argument method is not implemented for " "CategoricalIndex.reindex"
+            )
         if level is not None:
-            raise NotImplementedError("argument level is not implemented for "
-                                      "CategoricalIndex.reindex")
+            raise NotImplementedError(
+                "argument level is not implemented for " "CategoricalIndex.reindex"
+            )
         if limit is not None:
-            raise NotImplementedError("argument limit is not implemented for "
-                                      "CategoricalIndex.reindex")
+            raise NotImplementedError(
+                "argument limit is not implemented for " "CategoricalIndex.reindex"
+            )
 
-        target = ibase._ensure_index(target)
+        target = ibase.ensure_index(target)
 
-        if not is_categorical_dtype(target) and not target.is_unique:
-            raise ValueError("cannot reindex with a non-unique indexer")
+        if self.equals(target):
+            indexer = None
+            missing = []
+        else:
+            if not target.is_unique:
+                raise ValueError("cannot reindex with a non-unique indexer")
 
-        indexer, missing = self.get_indexer_non_unique(np.array(target))
+            indexer, missing = self.get_indexer_non_unique(np.array(target))
 
-        if len(self.codes):
+        if len(self.codes) and indexer is not None:
             new_target = self.take(indexer)
         else:
             new_target = target
@@ -516,8 +617,7 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
             if (cats == -1).any():
                 # coerce to a regular index here!
                 result = Index(np.array(self), name=self.name)
-                new_target, indexer, _ = result._reindex_non_unique(
-                    np.array(target))
+                new_target, indexer, _ = result._reindex_non_unique(np.array(target))
             else:
 
                 codes = new_target.codes.copy()
@@ -557,32 +657,34 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
 
         return new_target, indexer, new_indexer
 
-    @Appender(_index_shared_docs['get_indexer'] % _index_doc_kwargs)
+    @Appender(_index_shared_docs["get_indexer"] % _index_doc_kwargs)
     def get_indexer(self, target, method=None, limit=None, tolerance=None):
         from pandas.core.arrays.categorical import _recode_for_categories
 
         method = missing.clean_reindex_fill_method(method)
-        target = ibase._ensure_index(target)
+        target = ibase.ensure_index(target)
 
         if self.is_unique and self.equals(target):
-            return np.arange(len(self), dtype='intp')
+            return np.arange(len(self), dtype="intp")
 
-        if method == 'pad' or method == 'backfill':
-            raise NotImplementedError("method='pad' and method='backfill' not "
-                                      "implemented yet for CategoricalIndex")
-        elif method == 'nearest':
-            raise NotImplementedError("method='nearest' not implemented yet "
-                                      'for CategoricalIndex')
+        if method == "pad" or method == "backfill":
+            raise NotImplementedError(
+                "method='pad' and method='backfill' not "
+                "implemented yet for CategoricalIndex"
+            )
+        elif method == "nearest":
+            raise NotImplementedError(
+                "method='nearest' not implemented yet " "for CategoricalIndex"
+            )
 
-        if (isinstance(target, CategoricalIndex) and
-                self.values.is_dtype_equal(target)):
+        if isinstance(target, CategoricalIndex) and self.values.is_dtype_equal(target):
             if self.values.equals(target.values):
                 # we have the same codes
                 codes = target.codes
             else:
-                codes = _recode_for_categories(target.codes,
-                                               target.categories,
-                                               self.values.categories)
+                codes = _recode_for_categories(
+                    target.codes, target.categories, self.values.categories
+                )
         else:
             if isinstance(target, CategoricalIndex):
                 code_indexer = self.categories.get_indexer(target.categories)
@@ -591,28 +693,32 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
                 codes = self.categories.get_indexer(target)
 
         indexer, _ = self._engine.get_indexer_non_unique(codes)
-        return _ensure_platform_int(indexer)
+        return ensure_platform_int(indexer)
 
-    @Appender(_index_shared_docs['get_indexer_non_unique'] % _index_doc_kwargs)
+    @Appender(_index_shared_docs["get_indexer_non_unique"] % _index_doc_kwargs)
     def get_indexer_non_unique(self, target):
-        target = ibase._ensure_index(target)
+        target = ibase.ensure_index(target)
 
         if isinstance(target, CategoricalIndex):
-            target = target.categories
+            # Indexing on codes is more efficient if categories are the same:
+            if target.categories is self.categories:
+                target = target.codes
+                indexer, missing = self._engine.get_indexer_non_unique(target)
+                return ensure_platform_int(indexer), missing
+            target = target.values
 
         codes = self.categories.get_indexer(target)
         indexer, missing = self._engine.get_indexer_non_unique(codes)
-        return _ensure_platform_int(indexer), missing
+        return ensure_platform_int(indexer), missing
 
-    @Appender(_index_shared_docs['_convert_scalar_indexer'])
+    @Appender(_index_shared_docs["_convert_scalar_indexer"])
     def _convert_scalar_indexer(self, key, kind=None):
         if self.categories._defer_to_indexing:
             return self.categories._convert_scalar_indexer(key, kind=kind)
 
-        return super(CategoricalIndex, self)._convert_scalar_indexer(
-            key, kind=kind)
+        return super()._convert_scalar_indexer(key, kind=kind)
 
-    @Appender(_index_shared_docs['_convert_list_indexer'])
+    @Appender(_index_shared_docs["_convert_list_indexer"])
     def _convert_list_indexer(self, keyarr, kind=None):
         # Return our indexer or raise if all of the values are not included in
         # the categories
@@ -626,32 +732,35 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
             raise KeyError(
                 "a list-indexer must only "
                 "include values that are "
-                "in the categories")
+                "in the categories"
+            )
 
         return self.get_indexer(keyarr)
 
-    @Appender(_index_shared_docs['_convert_arr_indexer'])
+    @Appender(_index_shared_docs["_convert_arr_indexer"])
     def _convert_arr_indexer(self, keyarr):
-        keyarr = com._asarray_tuplesafe(keyarr)
+        keyarr = com.asarray_tuplesafe(keyarr)
 
         if self.categories._defer_to_indexing:
             return keyarr
 
         return self._shallow_copy(keyarr)
 
-    @Appender(_index_shared_docs['_convert_index_indexer'])
+    @Appender(_index_shared_docs["_convert_index_indexer"])
     def _convert_index_indexer(self, keyarr):
         return self._shallow_copy(keyarr)
 
-    @Appender(_index_shared_docs['take'] % _index_doc_kwargs)
-    def take(self, indices, axis=0, allow_fill=True,
-             fill_value=None, **kwargs):
+    @Appender(_index_shared_docs["take"] % _index_doc_kwargs)
+    def take(self, indices, axis=0, allow_fill=True, fill_value=None, **kwargs):
         nv.validate_take(tuple(), kwargs)
-        indices = _ensure_platform_int(indices)
-        taken = self._assert_take_fillable(self.codes, indices,
-                                           allow_fill=allow_fill,
-                                           fill_value=fill_value,
-                                           na_value=-1)
+        indices = ensure_platform_int(indices)
+        taken = self._assert_take_fillable(
+            self.codes,
+            indices,
+            allow_fill=allow_fill,
+            fill_value=fill_value,
+            na_value=-1,
+        )
         return self._create_from_codes(taken)
 
     def is_dtype_equal(self, other):
@@ -759,8 +868,10 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
         """
         code = self.categories.get_indexer([item])
         if (code == -1) and not (is_scalar(item) and isna(item)):
-            raise TypeError("cannot insert an item into a CategoricalIndex "
-                            "that is not already an existing category")
+            raise TypeError(
+                "cannot insert an item into a CategoricalIndex "
+                "that is not already an existing category"
+            )
 
         codes = self.codes
         codes = np.concatenate((codes[:loc], code, codes[loc:]))
@@ -775,8 +886,7 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
         Concatenate to_concat which has the same class
         ValueError if other is not in the categories
         """
-        to_concat = [self._is_dtype_compat(c) for c in to_concat]
-        codes = np.concatenate([c.codes for c in to_concat])
+        codes = np.concatenate([self._is_dtype_compat(c).codes for c in to_concat])
         result = self._create_from_codes(codes, name=name)
         # if name is None, _create_from_codes sets self.name
         result.name = name
@@ -791,7 +901,7 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
         """ add in comparison methods """
 
         def _make_compare(op):
-            opname = '__{op}__'.format(op=op.__name__)
+            opname = "__{op}__".format(op=op.__name__)
 
             def _evaluate_compare(self, other):
 
@@ -800,20 +910,19 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
                 if isinstance(other, CategoricalIndex):
                     other = other._values
                 elif isinstance(other, Index):
-                    other = self._create_categorical(
-                        self, other._values, categories=self.categories,
-                        ordered=self.ordered)
+                    other = self._create_categorical(other._values, dtype=self.dtype)
 
-                if isinstance(other, (ABCCategorical, np.ndarray,
-                                      ABCSeries)):
+                if isinstance(other, (ABCCategorical, np.ndarray, ABCSeries)):
                     if len(self.values) != len(other):
                         raise ValueError("Lengths must match to compare")
 
                 if isinstance(other, ABCCategorical):
                     if not self.values.is_dtype_equal(other):
-                        raise TypeError("categorical index comparisons must "
-                                        "have the same categories and ordered "
-                                        "attributes")
+                        raise TypeError(
+                            "categorical index comparisons must "
+                            "have the same categories and ordered "
+                            "attributes"
+                        )
 
                 result = op(self.values, other)
                 if isinstance(result, ABCSeries):
@@ -834,32 +943,15 @@ class CategoricalIndex(Index, accessor.PandasDelegate):
     def _delegate_method(self, name, *args, **kwargs):
         """ method delegation to the ._values """
         method = getattr(self._values, name)
-        if 'inplace' in kwargs:
+        if "inplace" in kwargs:
             raise ValueError("cannot use inplace with CategoricalIndex")
         res = method(*args, **kwargs)
         if is_scalar(res):
             return res
         return CategoricalIndex(res, name=self.name)
 
-    @classmethod
-    def _add_accessors(cls):
-        """ add in Categorical accessor methods """
-
-        from pandas.core.arrays import Categorical
-        CategoricalIndex._add_delegate_accessors(
-            delegate=Categorical, accessors=["rename_categories",
-                                             "reorder_categories",
-                                             "add_categories",
-                                             "remove_categories",
-                                             "remove_unused_categories",
-                                             "set_categories",
-                                             "as_ordered", "as_unordered",
-                                             "min", "max"],
-            typ='method', overwrite=True)
-
 
 CategoricalIndex._add_numeric_methods_add_sub_disabled()
 CategoricalIndex._add_numeric_methods_disabled()
 CategoricalIndex._add_logical_methods_disabled()
 CategoricalIndex._add_comparison_methods()
-CategoricalIndex._add_accessors()
