@@ -28,6 +28,7 @@ class FrozenList(PandasObject, list):
     because it's technically non-hashable, will be used
     for lookups, appropriately, etc.
     """
+
     # Side note: This has to be of type list. Otherwise,
     #            it messes up PyTables type checks.
 
@@ -47,7 +48,7 @@ class FrozenList(PandasObject, list):
         """
         if isinstance(other, tuple):
             other = list(other)
-        return type(self)(super(FrozenList, self).__add__(other))
+        return type(self)(super().__add__(other))
 
     def difference(self, other):
         """
@@ -72,13 +73,13 @@ class FrozenList(PandasObject, list):
 
     # Python 2 compat
     def __getslice__(self, i, j):
-        return self.__class__(super(FrozenList, self).__getslice__(i, j))
+        return self.__class__(super().__getslice__(i, j))
 
     def __getitem__(self, n):
         # Python 3 compat
         if isinstance(n, slice):
-            return self.__class__(super(FrozenList, self).__getitem__(n))
-        return super(FrozenList, self).__getitem__(n)
+            return self.__class__(super().__getitem__(n))
+        return super().__getitem__(n)
 
     def __radd__(self, other):
         if isinstance(other, tuple):
@@ -88,12 +89,12 @@ class FrozenList(PandasObject, list):
     def __eq__(self, other):
         if isinstance(other, (tuple, FrozenList)):
             other = list(other)
-        return super(FrozenList, self).__eq__(other)
+        return super().__eq__(other)
 
     __req__ = __eq__
 
     def __mul__(self, other):
-        return self.__class__(super(FrozenList, self).__mul__(other))
+        return self.__class__(super().__mul__(other))
 
     __imul__ = __mul__
 
@@ -105,16 +106,15 @@ class FrozenList(PandasObject, list):
 
     def _disabled(self, *args, **kwargs):
         """This method will not function because object is immutable."""
-        raise TypeError("'%s' does not support mutable operations." %
-                        self.__class__.__name__)
+        raise TypeError(
+            "'%s' does not support mutable operations." % self.__class__.__name__
+        )
 
-    def __unicode__(self):
-        return pprint_thing(self, quote_strings=True,
-                            escape_chars=('\t', '\r', '\n'))
+    def __str__(self):
+        return pprint_thing(self, quote_strings=True, escape_chars=("\t", "\r", "\n"))
 
     def __repr__(self):
-        return "%s(%s)" % (self.__class__.__name__,
-                           str(self))
+        return "%s(%s)" % (self.__class__.__name__, str(self))
 
     __setitem__ = __setslice__ = __delitem__ = __delslice__ = _disabled
     pop = append = extend = remove = sort = insert = _disabled
@@ -124,9 +124,12 @@ class FrozenNDArray(PandasObject, np.ndarray):
 
     # no __array_finalize__ for now because no metadata
     def __new__(cls, data, dtype=None, copy=False):
-        warnings.warn("\nFrozenNDArray is deprecated and will be removed in a "
-                      "future version.\nPlease use `numpy.ndarray` instead.\n",
-                      FutureWarning, stacklevel=2)
+        warnings.warn(
+            "\nFrozenNDArray is deprecated and will be removed in a "
+            "future version.\nPlease use `numpy.ndarray` instead.\n",
+            FutureWarning,
+            stacklevel=2,
+        )
 
         if copy is None:
             copy = not isinstance(data, FrozenNDArray)
@@ -135,8 +138,7 @@ class FrozenNDArray(PandasObject, np.ndarray):
 
     def _disabled(self, *args, **kwargs):
         """This method will not function because object is immutable."""
-        raise TypeError("'%s' does not support mutable operations." %
-                        self.__class__)
+        raise TypeError("'%s' does not support mutable operations." % self.__class__)
 
     __setitem__ = __setslice__ = __delitem__ = __delslice__ = _disabled
     put = itemset = fill = _disabled
@@ -149,15 +151,11 @@ class FrozenNDArray(PandasObject, np.ndarray):
         arr = self.view(np.ndarray).copy()
         return arr
 
-    def __unicode__(self):
+    def __repr__(self):
         """
         Return a string representation for this object.
-
-        Invoked by unicode(df) in py2 only. Yields a Unicode String in both
-        py2/py3.
         """
-        prepr = pprint_thing(self, escape_chars=('\t', '\r', '\n'),
-                             quote_strings=True)
+        prepr = pprint_thing(self, escape_chars=("\t", "\r", "\n"), quote_strings=True)
         return "%s(%s, dtype='%s')" % (type(self).__name__, prepr, self.dtype)
 
     @deprecate_kwarg(old_arg_name="v", new_arg_name="value")
@@ -184,8 +182,7 @@ class FrozenNDArray(PandasObject, np.ndarray):
         except ValueError:
             pass
 
-        return super(FrozenNDArray, self).searchsorted(
-            value, side=side, sorter=sorter)
+        return super().searchsorted(value, side=side, sorter=sorter)
 
 
 def _ensure_frozen(array_like, categories, copy=False):
