@@ -1134,6 +1134,17 @@ class TestTypeInference:
         result = lib.infer_dtype(Series(arr), skipna=True)
         assert result == "categorical"
 
+    def test_interval(self):
+        idx = pd.IntervalIndex.from_breaks(range(5), closed="both")
+        inferred = lib.infer_dtype(idx, skipna=False)
+        assert inferred == "interval"
+
+        inferred = lib.infer_dtype(idx._data, skipna=False)
+        assert inferred == "interval"
+
+        inferred = lib.infer_dtype(pd.Series(idx), skipna=False)
+        assert inferred == "interval"
+
 
 class TestNumberScalar:
     def test_is_number(self):
