@@ -1,100 +1,68 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-import pytest
 import os
 import warnings
 
-from pandas import DataFrame, Series
-from pandas.compat import zip, iteritems
-from pandas.util._decorators import cache_readonly
-from pandas.core.dtypes.api import is_list_like
-import pandas.util.testing as tm
-from pandas.util.testing import (ensure_clean,
-                                 assert_is_valid_plot_return_object)
-
 import numpy as np
 from numpy import random
 
-import pandas.plotting as plotting
-from pandas.plotting._tools import _flatten
+from pandas.util._decorators import cache_readonly
+import pandas.util._test_decorators as td
+
+from pandas.core.dtypes.api import is_list_like
+
+from pandas import DataFrame, Series
+import pandas.util.testing as tm
+from pandas.util.testing import assert_is_valid_plot_return_object, ensure_clean
+
 
 """
 This is a common base class used for various plotting tests
 """
 
-tm._skip_if_no_mpl()
-
-
-def _skip_if_no_scipy_gaussian_kde():
-    try:
-        from scipy.stats import gaussian_kde  # noqa
-    except ImportError:
-        pytest.skip("scipy version doesn't support gaussian_kde")
-
-
-def _ok_for_gaussian_kde(kind):
-    if kind in ['kde', 'density']:
-        try:
-            from scipy.stats import gaussian_kde  # noqa
-        except ImportError:
-            return False
-
-    return plotting._compat._mpl_ge_1_5_0()
-
-
-class TestPlotBase(object):
 
+@td.skip_if_no_mpl
+class TestPlotBase:
     def setup_method(self, method):
 
         import matplotlib as mpl
+        from pandas.plotting._matplotlib import compat
+
         mpl.rcdefaults()
 
-        self.mpl_le_1_2_1 = plotting._compat._mpl_le_1_2_1()
-        self.mpl_ge_1_3_1 = plotting._compat._mpl_ge_1_3_1()
-        self.mpl_ge_1_4_0 = plotting._compat._mpl_ge_1_4_0()
-        self.mpl_ge_1_5_0 = plotting._compat._mpl_ge_1_5_0()
-        self.mpl_ge_2_0_0 = plotting._compat._mpl_ge_2_0_0()
-        self.mpl_ge_2_0_1 = plotting._compat._mpl_ge_2_0_1()
+        self.mpl_ge_2_2_3 = compat._mpl_ge_2_2_3()
+        self.mpl_ge_3_0_0 = compat._mpl_ge_3_0_0()
+        self.mpl_ge_3_1_0 = compat._mpl_ge_3_1_0()
 
-        if self.mpl_ge_1_4_0:
-            self.bp_n_objects = 7
-        else:
-            self.bp_n_objects = 8
-        if self.mpl_ge_1_5_0:
-            # 1.5 added PolyCollections to legend handler
-            # so we have twice as many items.
-            self.polycollection_factor = 2
-        else:
-            self.polycollection_factor = 1
-
-        if self.mpl_ge_2_0_0:
-            self.default_figsize = (6.4, 4.8)
-        else:
-            self.default_figsize = (8.0, 6.0)
-        self.default_tick_position = 'left' if self.mpl_ge_2_0_0 else 'default'
-        # common test data
-        from pandas import read_csv
-        base = os.path.join(os.path.dirname(curpath()), os.pardir)
-        path = os.path.join(base, 'tests', 'data', 'iris.csv')
-        self.iris = read_csv(path)
+        self.bp_n_objects = 7
+        self.polycollection_factor = 2
+        self.default_figsize = (6.4, 4.8)
+        self.default_tick_position = "left"
 
         n = 100
         with tm.RNGContext(42):
-            gender = np.random.choice(['Male', 'Female'], size=n)
-            classroom = np.random.choice(['A', 'B', 'C'], size=n)
+            gender = np.random.choice(["Male", "Female"], size=n)
+            classroom = np.random.choice(["A", "B", "C"], size=n)
 
-            self.hist_df = DataFrame({'gender': gender,
-                                      'classroom': classroom,
-                                      'height': random.normal(66, 4, size=n),
-                                      'weight': random.normal(161, 32, size=n),
-                                      'category': random.randint(4, size=n)})
+            self.hist_df = DataFrame(
+                {
+                    "gender": gender,
+                    "classroom": classroom,
+                    "height": random.normal(66, 4, size=n),
+                    "weight": random.normal(161, 32, size=n),
+                    "category": random.randint(4, size=n),
+                }
+            )
 
         self.tdf = tm.makeTimeDataFrame()
-        self.hexbin_df = DataFrame({"A": np.random.uniform(size=20),
-                                    "B": np.random.uniform(size=20),
-                                    "C": np.arange(20) + np.random.uniform(
-                                        size=20)})
+        self.hexbin_df = DataFrame(
+            {
+                "A": np.random.uniform(size=20),
+                "B": np.random.uniform(size=20),
+                "C": np.arange(20) + np.random.uniform(size=20),
+            }
+        )
 
     def teardown_method(self, method):
         tm.close()
@@ -102,11 +70,13 @@ class TestPlotBase(object):
     @cache_readonly
     def plt(self):
         import matplotlib.pyplot as plt
+
         return plt
 
     @cache_readonly
     def colorconverter(self):
         import matplotlib.colors as colors
+
         return colors.colorConverter
 
     def _check_legend_labels(self, axes, labels=None, visible=True):
@@ -124,7 +94,7 @@ class TestPlotBase(object):
         """
 
         if visible and (labels is None):
-            raise ValueError('labels must be specified when visible is True')
+            raise ValueError("labels must be specified when visible is True")
         axes = self._flatten_visible(axes)
         for ax in axes:
             if visible:
@@ -166,8 +136,8 @@ class TestPlotBase(object):
             expected visibility
         """
         from matplotlib.collections import Collection
-        if not isinstance(collections,
-                          Collection) and not is_list_like(collections):
+
+        if not isinstance(collections, Collection) and not is_list_like(collections):
             collections = [collections]
 
         for patch in collections:
@@ -180,8 +150,9 @@ class TestPlotBase(object):
         mapped = dict(zip(unique, colors))
         return [mapped[v] for v in series.values]
 
-    def _check_colors(self, collections, linecolors=None, facecolors=None,
-                      mapping=None):
+    def _check_colors(
+        self, collections, linecolors=None, facecolors=None, mapping=None
+    ):
         """
         Check each artist has expected line colors and face colors
 
@@ -199,15 +170,14 @@ class TestPlotBase(object):
         """
 
         from matplotlib.lines import Line2D
-        from matplotlib.collections import (
-            Collection, PolyCollection, LineCollection
-        )
+        from matplotlib.collections import Collection, PolyCollection, LineCollection
+
         conv = self.colorconverter
         if linecolors is not None:
 
             if mapping is not None:
                 linecolors = self._get_colors_mapped(mapping, linecolors)
-                linecolors = linecolors[:len(collections)]
+                linecolors = linecolors[: len(collections)]
 
             assert len(collections) == len(linecolors)
             for patch, color in zip(collections, linecolors):
@@ -227,7 +197,7 @@ class TestPlotBase(object):
 
             if mapping is not None:
                 facecolors = self._get_colors_mapped(mapping, facecolors)
-                facecolors = facecolors[:len(collections)]
+                facecolors = facecolors[: len(collections)]
 
             assert len(collections) == len(facecolors)
             for patch, color in zip(collections, facecolors):
@@ -259,11 +229,12 @@ class TestPlotBase(object):
         else:
             labels = [t.get_text() for t in texts]
             assert len(labels) == len(expected)
-            for l, e in zip(labels, expected):
-                assert l == e
+            for label, e in zip(labels, expected):
+                assert label == e
 
-    def _check_ticks_props(self, axes, xlabelsize=None, xrot=None,
-                           ylabelsize=None, yrot=None):
+    def _check_ticks_props(
+        self, axes, xlabelsize=None, xrot=None, ylabelsize=None, yrot=None
+    ):
         """
         Check each axes has expected tick properties
 
@@ -280,6 +251,7 @@ class TestPlotBase(object):
             expected yticks rotation
         """
         from matplotlib.ticker import NullFormatter
+
         axes = self._flatten_visible(axes)
         for ax in axes:
             if xlabelsize or xrot:
@@ -288,13 +260,11 @@ class TestPlotBase(object):
                     # retained
                     labels = ax.get_xticklabels()
                 else:
-                    labels = ax.get_xticklabels() + ax.get_xticklabels(
-                        minor=True)
+                    labels = ax.get_xticklabels() + ax.get_xticklabels(minor=True)
 
                 for label in labels:
                     if xlabelsize is not None:
-                        tm.assert_almost_equal(label.get_fontsize(),
-                                               xlabelsize)
+                        tm.assert_almost_equal(label.get_fontsize(), xlabelsize)
                     if xrot is not None:
                         tm.assert_almost_equal(label.get_rotation(), xrot)
 
@@ -302,17 +272,15 @@ class TestPlotBase(object):
                 if isinstance(ax.yaxis.get_minor_formatter(), NullFormatter):
                     labels = ax.get_yticklabels()
                 else:
-                    labels = ax.get_yticklabels() + ax.get_yticklabels(
-                        minor=True)
+                    labels = ax.get_yticklabels() + ax.get_yticklabels(minor=True)
 
                 for label in labels:
                     if ylabelsize is not None:
-                        tm.assert_almost_equal(label.get_fontsize(),
-                                               ylabelsize)
+                        tm.assert_almost_equal(label.get_fontsize(), ylabelsize)
                     if yrot is not None:
                         tm.assert_almost_equal(label.get_rotation(), yrot)
 
-    def _check_ax_scales(self, axes, xaxis='linear', yaxis='linear'):
+    def _check_ax_scales(self, axes, xaxis="linear", yaxis="linear"):
         """
         Check each axes has expected scales
 
@@ -329,8 +297,7 @@ class TestPlotBase(object):
             assert ax.xaxis.get_scale() == xaxis
             assert ax.yaxis.get_scale() == yaxis
 
-    def _check_axes_shape(self, axes, axes_num=None, layout=None,
-                          figsize=None):
+    def _check_axes_shape(self, axes, axes_num=None, layout=None, figsize=None):
         """
         Check expected number of axes is drawn in expected layout
 
@@ -345,6 +312,8 @@ class TestPlotBase(object):
         figsize : tuple
             expected figsize. default is matplotlib default
         """
+        from pandas.plotting._matplotlib.tools import _flatten
+
         if figsize is None:
             figsize = self.default_figsize
         visible_axes = self._flatten_visible(axes)
@@ -361,7 +330,8 @@ class TestPlotBase(object):
 
         tm.assert_numpy_array_equal(
             visible_axes[0].figure.get_size_inches(),
-            np.array(figsize, dtype=np.float64))
+            np.array(figsize, dtype=np.float64),
+        )
 
     def _get_axes_layout(self, axes):
         x_set = set()
@@ -382,6 +352,8 @@ class TestPlotBase(object):
         axes : matplotlib Axes object, or its list-like
 
         """
+        from pandas.plotting._matplotlib.tools import _flatten
+
         axes = _flatten(axes)
         axes = [ax for ax in axes if ax.get_visible()]
         return axes
@@ -404,8 +376,8 @@ class TestPlotBase(object):
             xerr_count = 0
             yerr_count = 0
             for c in containers:
-                has_xerr = getattr(c, 'has_xerr', False)
-                has_yerr = getattr(c, 'has_yerr', False)
+                has_xerr = getattr(c, "has_xerr", False)
+                has_yerr = getattr(c, "has_yerr", False)
                 if has_xerr:
                     xerr_count += 1
                 if has_yerr:
@@ -413,8 +385,9 @@ class TestPlotBase(object):
             assert xerr == xerr_count
             assert yerr == yerr_count
 
-    def _check_box_return_type(self, returned, return_type, expected_keys=None,
-                               check_ax_title=True):
+    def _check_box_return_type(
+        self, returned, return_type, expected_keys=None, check_ax_title=True
+    ):
         """
         Check box returned type is correct
 
@@ -432,14 +405,15 @@ class TestPlotBase(object):
             Normal ``plot`` doesn't attach ``ax.title``, it must be disabled.
         """
         from matplotlib.axes import Axes
-        types = {'dict': dict, 'axes': Axes, 'both': tuple}
+
+        types = {"dict": dict, "axes": Axes, "both": tuple}
         if expected_keys is None:
             # should be fixed when the returning default is changed
             if return_type is None:
-                return_type = 'dict'
+                return_type = "dict"
 
             assert isinstance(returned, types[return_type])
-            if return_type == 'both':
+            if return_type == "both":
                 assert isinstance(returned.ax, Axes)
                 assert isinstance(returned.lines, dict)
         else:
@@ -452,20 +426,20 @@ class TestPlotBase(object):
             assert isinstance(returned, Series)
 
             assert sorted(returned.keys()) == sorted(expected_keys)
-            for key, value in iteritems(returned):
+            for key, value in returned.items():
                 assert isinstance(value, types[return_type])
                 # check returned dict has correct mapping
-                if return_type == 'axes':
+                if return_type == "axes":
                     if check_ax_title:
                         assert value.get_title() == key
-                elif return_type == 'both':
+                elif return_type == "both":
                     if check_ax_title:
                         assert value.ax.get_title() == key
                     assert isinstance(value.ax, Axes)
                     assert isinstance(value.lines, dict)
-                elif return_type == 'dict':
-                    line = value['medians'][0]
-                    axes = line.axes if self.mpl_ge_1_5_0 else line.get_axes()
+                elif return_type == "dict":
+                    line = value["medians"][0]
+                    axes = line.axes
                     if check_ax_title:
                         assert axes.get_title() == key
                 else:
@@ -477,77 +451,76 @@ class TestPlotBase(object):
         import matplotlib as mpl
 
         def is_grid_on():
-            xoff = all(not g.gridOn
-                       for g in self.plt.gca().xaxis.get_major_ticks())
-            yoff = all(not g.gridOn
-                       for g in self.plt.gca().yaxis.get_major_ticks())
+            xticks = self.plt.gca().xaxis.get_major_ticks()
+            yticks = self.plt.gca().yaxis.get_major_ticks()
+            # for mpl 2.2.2, gridOn and gridline.get_visible disagree.
+            # for new MPL, they are the same.
+
+            if self.mpl_ge_3_1_0:
+                xoff = all(not g.gridline.get_visible() for g in xticks)
+                yoff = all(not g.gridline.get_visible() for g in yticks)
+            else:
+                xoff = all(not g.gridOn for g in xticks)
+                yoff = all(not g.gridOn for g in yticks)
+
             return not (xoff and yoff)
 
         spndx = 1
         for kind in kinds:
-            if not _ok_for_gaussian_kde(kind):
-                continue
 
             self.plt.subplot(1, 4 * len(kinds), spndx)
             spndx += 1
-            mpl.rc('axes', grid=False)
+            mpl.rc("axes", grid=False)
             obj.plot(kind=kind, **kws)
             assert not is_grid_on()
 
             self.plt.subplot(1, 4 * len(kinds), spndx)
             spndx += 1
-            mpl.rc('axes', grid=True)
+            mpl.rc("axes", grid=True)
             obj.plot(kind=kind, grid=False, **kws)
             assert not is_grid_on()
 
-            if kind != 'pie':
+            if kind != "pie":
                 self.plt.subplot(1, 4 * len(kinds), spndx)
                 spndx += 1
-                mpl.rc('axes', grid=True)
+                mpl.rc("axes", grid=True)
                 obj.plot(kind=kind, **kws)
                 assert is_grid_on()
 
                 self.plt.subplot(1, 4 * len(kinds), spndx)
                 spndx += 1
-                mpl.rc('axes', grid=False)
+                mpl.rc("axes", grid=False)
                 obj.plot(kind=kind, grid=True, **kws)
                 assert is_grid_on()
 
-    def _maybe_unpack_cycler(self, rcParams, field='color'):
+    def _unpack_cycler(self, rcParams, field="color"):
         """
-        Compat layer for MPL 1.5 change to color cycle
-
-        Before: plt.rcParams['axes.color_cycle'] -> ['b', 'g', 'r'...]
-        After : plt.rcParams['axes.prop_cycle'] -> cycler(...)
+        Auxiliary function for correctly unpacking cycler after MPL >= 1.5
         """
-        if self.mpl_ge_1_5_0:
-            cyl = rcParams['axes.prop_cycle']
-            colors = [v[field] for v in cyl]
-        else:
-            colors = rcParams['axes.color_cycle']
-        return colors
+        return [v[field] for v in rcParams["axes.prop_cycle"]]
 
 
-def _check_plot_works(f, filterwarnings='always', **kwargs):
+def _check_plot_works(f, filterwarnings="always", **kwargs):
     import matplotlib.pyplot as plt
+
     ret = None
     with warnings.catch_warnings():
         warnings.simplefilter(filterwarnings)
         try:
             try:
-                fig = kwargs['figure']
+                fig = kwargs["figure"]
             except KeyError:
                 fig = plt.gcf()
 
             plt.clf()
 
-            ax = kwargs.get('ax', fig.add_subplot(211))  # noqa
+            ax = kwargs.get("ax", fig.add_subplot(211))  # noqa
             ret = f(**kwargs)
 
             assert_is_valid_plot_return_object(ret)
 
             try:
-                kwargs['ax'] = fig.add_subplot(212)
+                kwargs["ax"] = fig.add_subplot(212)
                 ret = f(**kwargs)
             except Exception:
                 pass
