@@ -1,8 +1,15 @@
-from pandas import DateOffset, DatetimeIndex, Series, Timestamp
-from pandas.compat import add_metaclass
 from datetime import datetime, timedelta
-from dateutil.relativedelta import MO, TU, WE, TH, FR, SA, SU
-from pandas.tseries.offsets import Easter, Day
+from typing import List
+import warnings
+
+from dateutil.relativedelta import FR, MO, SA, SU, TH, TU, WE  # noqa
+import numpy as np
+
+from pandas.errors import PerformanceWarning
+
+from pandas import DateOffset, Series, Timestamp, date_range
+
+from pandas.tseries.offsets import Day, Easter
 
 
 def next_monday(dt):
@@ -16,6 +23,7 @@ def next_monday(dt):
         return dt + timedelta(1)
     return dt
 
+
 def next_monday_or_tuesday(dt):
     """
     For second holiday of two adjacent ones!
@@ -30,6 +38,7 @@ def next_monday_or_tuesday(dt):
         return dt + timedelta(1)
     return dt
 
+
 def previous_friday(dt):
     """
     If holiday falls on Saturday or Sunday, use previous Friday instead.
@@ -40,6 +49,7 @@ def previous_friday(dt):
         return dt - timedelta(2)
     return dt
 
+
 def sunday_to_monday(dt):
     """
     If holiday falls on Sunday, use day thereafter (Monday) instead.
@@ -111,14 +121,24 @@ def after_nearest_workday(dt):
     return next_workday(nearest_workday(dt))
 
 
-class Holiday(object):
+class Holiday:
     """
     Class that defines a holiday with start/end dates and rules
     for observance.
     """
-    def __init__(self, name, year=None, month=None, day=None, offset=None,
-                 observance=None, start_date=None, end_date=None,
-                 days_of_week=None):
+
+    def __init__(
+        self,
+        name,
+        year=None,
+        month=None,
+        day=None,
+        offset=None,
+        observance=None,
+        start_date=None,
+        end_date=None,
+        days_of_week=None,
+    ):
         """
         Parameters
         ----------
@@ -126,7 +146,7 @@ class Holiday(object):
             Name of the holiday , defaults to class name
         offset : array of pandas.tseries.offsets or
                 class from pandas.tseries.offsets
-            computes offset from  date
+            computes offset from date
         observance: function
             computes when holiday is given a pandas Timestamp
         days_of_week:
@@ -136,12 +156,11 @@ class Holiday(object):
         Examples
         --------
         >>> from pandas.tseries.holiday import Holiday, nearest_workday
-        >>> from pandas import DateOffset
         >>> from dateutil.relativedelta import MO
-        >>> USMemorialDay = Holiday('MemorialDay', month=5, day=24,
-                                    offset=DateOffset(weekday=MO(1)))
+        >>> USMemorialDay = Holiday('Memorial Day', month=5, day=31,
+                                    offset=pd.DateOffset(weekday=MO(-1)))
         >>> USLaborDay = Holiday('Labor Day', month=9, day=1,
-                            offset=DateOffset(weekday=MO(1)))
+                                offset=pd.DateOffset(weekday=MO(1)))
         >>> July3rd = Holiday('July 3rd', month=7, day=3,)
         >>> NewYears = Holiday('New Years Day', month=1,  day=1,
                                observance=nearest_workday),
@@ -156,30 +175,32 @@ class Holiday(object):
         self.month = month
         self.day = day
         self.offset = offset
-        self.start_date = start_date
-        self.end_date = end_date
+        self.start_date = (
+            Timestamp(start_date) if start_date is not None else start_date
+        )
+        self.end_date = Timestamp(end_date) if end_date is not None else end_date
         self.observance = observance
-        assert (days_of_week is None or type(days_of_week) == tuple)
+        assert days_of_week is None or type(days_of_week) == tuple
         self.days_of_week = days_of_week
 
     def __repr__(self):
-        info = ''
+        info = ""
         if self.year is not None:
-            info += 'year=%s, ' % self.year
-        info += 'month=%s, day=%s, ' % (self.month, self.day)
+            info += "year={year}, ".format(year=self.year)
+        info += "month={mon}, day={day}, ".format(mon=self.month, day=self.day)
 
         if self.offset is not None:
-            info += 'offset=%s' % self.offset
+            info += "offset={offset}".format(offset=self.offset)
 
         if self.observance is not None:
-            info += 'observance=%s' % self.observance
+            info += "observance={obs}".format(obs=self.observance)
 
-        repr = 'Holiday: %s (%s)' % (self.name, info)
+        repr = "Holiday: {name} ({info})".format(name=self.name, info=info)
         return repr
 
     def dates(self, start_date, end_date, return_name=False):
         """
-        Calculate holidays between start date and end date
+        Calculate holidays observed between start date and end date
 
         Parameters
         ----------
@@ -189,6 +210,12 @@ class Holiday(object):
             If True, return a series that has dates and holiday names.
             False will only return dates.
         """
+        start_date = Timestamp(start_date)
+        end_date = Timestamp(end_date)
+
+        filter_start_date = start_date
+        filter_end_date = end_date
+
         if self.year is not None:
             dt = Timestamp(datetime(self.year, self.month, self.day))
             if return_name:
@@ -196,40 +223,68 @@ class Holiday(object):
             else:
                 return [dt]
 
-        if self.start_date is not None:
-            start_date = self.start_date
-
-        if self.end_date is not None:
-            end_date = self.end_date
-
-        start_date = Timestamp(start_date)
-        end_date = Timestamp(end_date)
-
-        year_offset = DateOffset(years=1)
-        base_date = Timestamp(
-            datetime(start_date.year, self.month, self.day),
-            tz=start_date.tz,
-        )
-        dates = DatetimeIndex(start=base_date, end=end_date, freq=year_offset)
+        dates = self._reference_dates(start_date, end_date)
         holiday_dates = self._apply_rule(dates)
         if self.days_of_week is not None:
-            holiday_dates = list(filter(lambda x: x is not None and
-                                                  x.dayofweek in self.days_of_week,
-                                                  holiday_dates))
-        else:
-            holiday_dates = list(filter(lambda x: x is not None, holiday_dates))
+            holiday_dates = holiday_dates[
+                np.in1d(holiday_dates.dayofweek, self.days_of_week)
+            ]
+
+        if self.start_date is not None:
+            filter_start_date = max(
+                self.start_date.tz_localize(filter_start_date.tz), filter_start_date
+            )
+        if self.end_date is not None:
+            filter_end_date = min(
+                self.end_date.tz_localize(filter_end_date.tz), filter_end_date
+            )
+        holiday_dates = holiday_dates[
+            (holiday_dates >= filter_start_date) & (holiday_dates <= filter_end_date)
+        ]
         if return_name:
             return Series(self.name, index=holiday_dates)
         return holiday_dates
 
+    def _reference_dates(self, start_date, end_date):
+        """
+        Get reference dates for the holiday.
+
+        Return reference dates for the holiday also returning the year
+        prior to the start_date and year following the end_date.  This ensures
+        that any offsets to be applied will yield the holidays within
+        the passed in dates.
+        """
+        if self.start_date is not None:
+            start_date = self.start_date.tz_localize(start_date.tz)
+
+        if self.end_date is not None:
+            end_date = self.end_date.tz_localize(start_date.tz)
+
+        year_offset = DateOffset(years=1)
+        reference_start_date = Timestamp(
+            datetime(start_date.year - 1, self.month, self.day)
+        )
+
+        reference_end_date = Timestamp(
+            datetime(end_date.year + 1, self.month, self.day)
+        )
+        # Don't process unnecessary holidays
+        dates = date_range(
+            start=reference_start_date,
+            end=reference_end_date,
+            freq=year_offset,
+            tz=start_date.tz,
+        )
+
+        return dates
+
     def _apply_rule(self, dates):
         """
-        Apply the given offset/observance to an
-        iterable of dates.
+        Apply the given offset/observance to a DatetimeIndex of dates.
 
         Parameters
         ----------
-        dates : array-like
+        dates : DatetimeIndex
             Dates to apply the given offset/observance rule
 
         Returns
@@ -237,7 +292,7 @@ class Holiday(object):
         Dates with rules applied
         """
         if self.observance is not None:
-            return map(lambda d: self.observance(d), dates)
+            return dates.map(lambda d: self.observance(d))
 
         if self.offset is not None:
             if not isinstance(self.offset, list):
@@ -245,17 +300,26 @@ class Holiday(object):
             else:
                 offsets = self.offset
             for offset in offsets:
-                dates = list(map(lambda d: d + offset, dates))
+
+                # if we are adding a non-vectorized value
+                # ignore the PerformanceWarnings:
+                with warnings.catch_warnings():
+                    warnings.simplefilter("ignore", PerformanceWarning)
+                    dates += offset
         return dates
 
+
 holiday_calendars = {}
+
+
 def register(cls):
     try:
         name = cls.name
-    except:
+    except AttributeError:
         name = cls.__name__
     holiday_calendars[name] = cls
 
+
 def get_calendar(name):
     """
     Return an instance of a calendar based on its name.
@@ -267,19 +331,20 @@ def get_calendar(name):
     """
     return holiday_calendars[name]()
 
+
 class HolidayCalendarMetaClass(type):
     def __new__(cls, clsname, bases, attrs):
-        calendar_class = super(HolidayCalendarMetaClass, cls).__new__(cls, clsname, bases, attrs)
+        calendar_class = super().__new__(cls, clsname, bases, attrs)
         register(calendar_class)
         return calendar_class
 
-@add_metaclass(HolidayCalendarMetaClass)
-class AbstractHolidayCalendar(object):
+
+class AbstractHolidayCalendar(metaclass=HolidayCalendarMetaClass):
     """
     Abstract interface to create holidays following certain rules.
     """
-    __metaclass__ = HolidayCalendarMetaClass
-    rules = []
+
+    rules = []  # type: List[Holiday]
     start_date = Timestamp(datetime(1970, 1, 1))
     end_date = Timestamp(datetime(2030, 12, 31))
     _cache = None
@@ -296,7 +361,7 @@ class AbstractHolidayCalendar(object):
         rules : array of Holiday objects
             A set of rules used to create the holidays.
         """
-        super(AbstractHolidayCalendar, self).__init__()
+        super().__init__()
         if name is None:
             name = self.__class__.__name__
         self.name = name
@@ -304,6 +369,13 @@ class AbstractHolidayCalendar(object):
         if rules is not None:
             self.rules = rules
 
+    def rule_from_name(self, name):
+        for rule in self.rules:
+            if rule.name == name:
+                return rule
+
+        return None
+
     def holidays(self, start=None, end=None, return_name=False):
         """
         Returns a curve with holidays between start_date and end_date
@@ -312,7 +384,7 @@ class AbstractHolidayCalendar(object):
         ----------
         start : starting date, datetime-like, optional
         end : ending date, datetime-like, optional
-        return_names : bool, optional
+        return_name : bool, optional
             If True, return a series that has dates and holiday names.
             False will only return a DatetimeIndex of dates.
 
@@ -321,8 +393,10 @@ class AbstractHolidayCalendar(object):
             DatetimeIndex of holidays
         """
         if self.rules is None:
-            raise Exception('Holiday Calendar %s does not have any '
-                            'rules specified' % self.name)
+            raise Exception(
+                "Holiday Calendar {name} does not have any "
+                "rules specified".format(name=self.name)
+            )
 
         if start is None:
             start = AbstractHolidayCalendar.start_date
@@ -334,7 +408,8 @@ class AbstractHolidayCalendar(object):
         end = Timestamp(end)
 
         holidays = None
-        # If we don't have a cache or the dates are outside the prior cache, we get them again
+        # If we don't have a cache or the dates are outside the prior cache, we
+        # get them again
         if self._cache is None or start < self._cache[0] or end > self._cache[1]:
             for rule in self.rules:
                 rule_holidays = rule.dates(start, end, return_name=True)
@@ -363,26 +438,28 @@ class AbstractHolidayCalendar(object):
 
         Parameters
         ----------
-        base : AbstractHolidayCalendar instance/subclass or array of Holiday objects
-        other : AbstractHolidayCalendar instance/subclass or array of Holiday objects
+        base : AbstractHolidayCalendar
+          instance/subclass or array of Holiday objects
+        other : AbstractHolidayCalendar
+          instance/subclass or array of Holiday objects
         """
         try:
             other = other.rules
-        except:
+        except AttributeError:
             pass
 
         if not isinstance(other, list):
             other = [other]
-        other_holidays = dict((holiday.name, holiday) for holiday in other)
+        other_holidays = {holiday.name: holiday for holiday in other}
 
         try:
             base = base.rules
-        except:
+        except AttributeError:
             pass
 
         if not isinstance(base, list):
             base = [base]
-        base_holidays = dict([(holiday.name, holiday) for holiday in base])
+        base_holidays = {holiday.name: holiday for holiday in base}
 
         other_holidays.update(base_holidays)
         return list(other_holidays.values())
@@ -405,18 +482,27 @@ class AbstractHolidayCalendar(object):
         else:
             return holidays
 
-USMemorialDay = Holiday('MemorialDay', month=5, day=31,
-                        offset=DateOffset(weekday=MO(-1)))
-USLaborDay = Holiday('Labor Day', month=9, day=1,
-                     offset=DateOffset(weekday=MO(1)))
-USColumbusDay = Holiday('Columbus Day', month=10, day=1,
-                        offset=DateOffset(weekday=MO(2)))
-USThanksgivingDay = Holiday('Thanksgiving', month=11, day=1,
-                            offset=DateOffset(weekday=TH(4)))
-USMartinLutherKingJr = Holiday('Dr. Martin Luther King Jr.', start_date=datetime(1986,1,1), month=1, day=1,
-                               offset=DateOffset(weekday=MO(3)))
-USPresidentsDay = Holiday('President''s Day', month=2, day=1,
-                          offset=DateOffset(weekday=MO(3)))
+
+USMemorialDay = Holiday(
+    "Memorial Day", month=5, day=31, offset=DateOffset(weekday=MO(-1))
+)
+USLaborDay = Holiday("Labor Day", month=9, day=1, offset=DateOffset(weekday=MO(1)))
+USColumbusDay = Holiday(
+    "Columbus Day", month=10, day=1, offset=DateOffset(weekday=MO(2))
+)
+USThanksgivingDay = Holiday(
+    "Thanksgiving", month=11, day=1, offset=DateOffset(weekday=TH(4))
+)
+USMartinLutherKingJr = Holiday(
+    "Martin Luther King Jr. Day",
+    start_date=datetime(1986, 1, 1),
+    month=1,
+    day=1,
+    offset=DateOffset(weekday=MO(3)),
+)
+USPresidentsDay = Holiday(
+    "Presidents Day", month=2, day=1, offset=DateOffset(weekday=MO(3))
+)
 GoodFriday = Holiday("Good Friday", month=1, day=1, offset=[Easter(), Day(-2)])
 
 EasterMonday = Holiday("Easter Monday", month=1, day=1, offset=[Easter(), Day(1)])
@@ -424,21 +510,24 @@ EasterMonday = Holiday("Easter Monday", month=1, day=1, offset=[Easter(), Day(1)
 
 class USFederalHolidayCalendar(AbstractHolidayCalendar):
     """
-    US Federal Government Holiday Calendar based on rules specified
-    by: https://www.opm.gov/policy-data-oversight/snow-dismissal-procedures/federal-holidays/
+    US Federal Government Holiday Calendar based on rules specified by:
+    https://www.opm.gov/policy-data-oversight/
+       snow-dismissal-procedures/federal-holidays/
     """
+
     rules = [
-        Holiday('New Years Day', month=1,  day=1,  observance=nearest_workday),
+        Holiday("New Years Day", month=1, day=1, observance=nearest_workday),
         USMartinLutherKingJr,
         USPresidentsDay,
         USMemorialDay,
-        Holiday('July 4th', month=7,  day=4,  observance=nearest_workday),
+        Holiday("July 4th", month=7, day=4, observance=nearest_workday),
         USLaborDay,
         USColumbusDay,
-        Holiday('Veterans Day', month=11, day=11, observance=nearest_workday),
+        Holiday("Veterans Day", month=11, day=11, observance=nearest_workday),
         USThanksgivingDay,
-        Holiday('Christmas', month=12, day=25, observance=nearest_workday)
-        ]
+        Holiday("Christmas", month=12, day=25, observance=nearest_workday),
+    ]
+
 
 def HolidayCalendarFactory(name, base, other, base_class=AbstractHolidayCalendar):
     rules = AbstractHolidayCalendar.merge_class(base, other)
