@@ -5,6 +5,6 @@
 # this builds the release cleanly & is building on the current checkout
 rm -rf dist
 git clean -xfd
-python setup.py clean
-python setup.py cython
-python setup.py sdist --formats=gztar
+python setup.py clean --quiet
+python setup.py cython --quiet
+python setup.py sdist --formats=gztar --quiet
